package com.igaworks.adbrixrm_flutter;

import android.util.Log;

import com.igaworks.v2.core.AdBrixRm;

import android.os.Handler;
import android.os.Looper;

import org.json.JSONException;
import org.json.JSONObject;

import io.adbrix.sdk.component.AbxLog;
import io.adbrix.sdk.domain.model.Empty;
import io.adbrix.sdk.domain.model.Result;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;

class AdbrixrmEventChannel implements EventChannel.StreamHandler, AdBrixRm.DeeplinkListener,
        AdBrixRm.DeferredDeeplinkListener, AdBrixRm.InAppMessageClickListener, AdBrixRm.onTouchLocalPushListener,
        AdBrixRm.onTouchRemotePushListener, AdBrixRm.LogListener, AdBrixRm.DfnInAppMessageAutoFetchListener {
    private static final String ARGUMENT_Deeplink = "com.adbrix.stream.deeplink";
    private static final String ARGUMENT_DeferredDeeplink = "com.adbrix.stream.deferredDeeplink";
    private static final String ARGUMENT_PushLocalCallback = "com.adbrix.stream.pushLocalCallBack";
    private static final String ARGUMENT_PushRemoteCallback = "com.adbrix.stream.pushRemoteCallback";
    private static final String ARGUMENT_OnReceiveInAppMessageClick = "com.adbrix.stream.onReceiveInAppMessageClick";
    private static final String ARGUMENT_DidFetchInAppMessage = "com.adbrix.stream.didFetchInAppMessage";
    private static final String ARGUMENT_DidPrintLog = "com.adbrix.stream.didPrintLog";

    private static final String EVENT_CHANNEL_Deeplink = "com.adbrix.eventChannel.deeplink";
    private static final String EVENT_CHANNEL_DeferredDeeplink = "com.adbrix.eventChannel.deferredDeeplink";
    private static final String EVENT_CHANNEL_PushLocalCallback = "com.adbrix.eventChannel.pushLocalCallBack";
    private static final String EVENT_CHANNEL_PushRemoteCallback = "com.adbrix.eventChannel.pushRemoteCallback";
    private static final String EVENT_CHANNEL_OnReceiveInAppMessageClick = "com.adbrix.eventChannel.onReceiveInAppMessageClick";
    private static final String EVENT_CHANNEL_DidFetchInAppMessage = "com.adbrix.eventChannel.didFetchInAppMessage";
    private static final String EVENT_CHANNEL_DidPrintLog = "com.adbrix.eventChannel.didPrintLog";

    private EventChannel eventChannelDeeplink;
    private EventChannel eventChannelDeferredDeeplink;
    private EventChannel eventChannelDidPrintLog;
    private EventChannel eventChannelPushLocalCallback;
    private EventChannel eventChannelPushRemoteCallback;
    private EventChannel eventChannelDidFetchInAppMessage;
    private EventChannel eventChannelOnReceiveInAppMessageClick;

    public static final String ERROR_CODE_SDK_ERROR = "-1";

    private EventChannel.EventSink sinkDeeplink;
    private EventChannel.EventSink sinkDeferredDeeplink;
    private EventChannel.EventSink sinkPushLocal;
    private EventChannel.EventSink sinkPushRemote;
    private EventChannel.EventSink sinkOnReceiveInAppMessage;
    private EventChannel.EventSink sinkDidFetchInAppMessage;
    private EventChannel.EventSink sinkDidPrintLog;

    private String initialDeeplink;
    private String initialPushData;

    private static final AdbrixrmEventChannel instance = new AdbrixrmEventChannel();

    private AdbrixrmEventChannel(){}

    public static AdbrixrmEventChannel shared() {
        return instance;
    }

    public void linkEventChannel(BinaryMessenger messenger) {
        this.eventChannelDeeplink = new EventChannel(messenger, EVENT_CHANNEL_Deeplink);
        this.eventChannelDeferredDeeplink = new EventChannel(messenger, EVENT_CHANNEL_DeferredDeeplink);
        this.eventChannelPushLocalCallback = new EventChannel(messenger, EVENT_CHANNEL_PushLocalCallback);
        this.eventChannelPushRemoteCallback = new EventChannel(messenger, EVENT_CHANNEL_PushRemoteCallback);
        this.eventChannelDidFetchInAppMessage = new EventChannel(messenger, EVENT_CHANNEL_DidFetchInAppMessage);
        this.eventChannelOnReceiveInAppMessageClick = new EventChannel(messenger, EVENT_CHANNEL_OnReceiveInAppMessageClick);
        this.eventChannelDidPrintLog = new EventChannel(messenger, EVENT_CHANNEL_DidPrintLog);
    }

    void setStreamHandler() {
        eventChannelDeeplink.setStreamHandler(this);
        eventChannelDeferredDeeplink.setStreamHandler(this);
        eventChannelPushLocalCallback.setStreamHandler(this);
        eventChannelPushRemoteCallback.setStreamHandler(this);
        eventChannelDidFetchInAppMessage.setStreamHandler(this);
        eventChannelOnReceiveInAppMessageClick.setStreamHandler(this);
        eventChannelDidPrintLog.setStreamHandler(this);
    }

    void unSetStreamHandler() {
        eventChannelDeeplink.setStreamHandler(null);
        eventChannelDeferredDeeplink.setStreamHandler(null);
        eventChannelPushLocalCallback.setStreamHandler(null);
        eventChannelPushRemoteCallback.setStreamHandler(null);
        eventChannelDidFetchInAppMessage.setStreamHandler(null);
        eventChannelOnReceiveInAppMessageClick.setStreamHandler(null);
        eventChannelDidPrintLog.setStreamHandler(null);
    }

    @Override
    public void onListen(Object arguments, final EventChannel.EventSink events) {
        String argument = arguments.toString();
        switch (argument) {
            case ARGUMENT_Deeplink:
                this.sinkDeeplink = events;
                if (initialDeeplink != null) {
                    if (sinkDeeplink != null) {
                        String _initialDeeplink = initialDeeplink;
                        sinkDeeplink.success(_initialDeeplink);
                        initialDeeplink = null;
                    }
                }
                break;
            case ARGUMENT_DeferredDeeplink:
                this.sinkDeferredDeeplink = events;
                break;
            case ARGUMENT_PushLocalCallback:
                this.sinkPushLocal = events;
                break;
            case ARGUMENT_PushRemoteCallback:
                this.sinkPushRemote = events;
                if (initialPushData != null) {
                    if (sinkPushRemote != null) {
                        String _initialPushData = initialPushData;
                        sinkPushRemote.success(_initialPushData);
                        initialPushData = null;
                    }
                }
                break;
            case ARGUMENT_DidFetchInAppMessage:
                this.sinkDidFetchInAppMessage = events;
                break;
            case ARGUMENT_OnReceiveInAppMessageClick:
                this.sinkOnReceiveInAppMessage = events;
                break;
            case ARGUMENT_DidPrintLog:
                this.sinkDidPrintLog = events;
                break;
        }

    }

    @Override
    public void onCancel(Object arguments) {
        this.sinkDeeplink = null;
        this.sinkDeferredDeeplink = null;
        this.sinkPushLocal = null;
        this.sinkPushRemote = null;
        this.sinkDidFetchInAppMessage = null;
        this.sinkOnReceiveInAppMessage = null;
        this.sinkDidPrintLog = null;
    }

    @Override
    public void onReceiveDeeplink(String s) {
        final String deeplink = s;
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                if (sinkDeeplink != null) {
                    sinkDeeplink.success(deeplink);
                } else {
                    initialDeeplink = deeplink;
                }
            }
        });
    }

    @Override
    public void onReceiveDeferredDeeplink(String s) {
        final String deferredDeeplink = s;
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                if (sinkDeferredDeeplink != null) {
                    sinkDeferredDeeplink.success(deferredDeeplink);
                }
            }
        });
    }

    @Override
    public void onFetchInAppMessage(Result<Empty> result) {
        final Boolean isSucceeded = result.isSucceeded();
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                if (sinkDidFetchInAppMessage != null) {
                    sinkDidFetchInAppMessage.success(isSucceeded);
                }
            }
        });
    }

    @Override
    public void onReceiveInAppMessageClick(String s, String s1, String s2, boolean b) {

        final JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("actionId", s);
            jsonObject.put("actionType", s1);
            jsonObject.put("actionArg", s2);
            jsonObject.put("isClosed", b);
        } catch (JSONException e) {
            AbxLog.e("onReceiveInAppMessageClick() parse error", e, false);
        }
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                if (sinkOnReceiveInAppMessage != null) {
                    sinkOnReceiveInAppMessage.success(jsonObject.toString());
                }
            }
        });
    }

    @Override
    public void onPrintLog(int i, String s) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("[");
        switch (i) {
            case Log.VERBOSE:
                stringBuilder.append("V");
                break;
            case Log.DEBUG:
                stringBuilder.append("D");
                break;
            case Log.INFO:
                stringBuilder.append("I");
                break;
            case Log.ASSERT:
                stringBuilder.append("A");
                break;
            case Log.WARN:
                stringBuilder.append("W");
                break;
            case Log.ERROR:
                stringBuilder.append("E");
                break;
        }
        stringBuilder.append("] ");
        stringBuilder.append(s);

        final String log = stringBuilder.toString();
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {

                if (sinkDidPrintLog != null) {
                    sinkDidPrintLog.success(log);
                }
            }
        });
    }

    @Override
    public void onTouchLocalPush(String s) {
        final String pushData = s;
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                if (sinkPushLocal != null) {
                    sinkPushLocal.success(pushData);
                }
            }
        });
    }

    @Override
    public void onTouchRemotePush(String s) {
        final String pushData = s;
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                if (sinkPushRemote != null) {
                    sinkPushRemote.success(pushData);
                } else {
                    initialPushData = pushData;
                }
            }
        });
    }
}
