package com.igaworks.adbrixrm_flutter;

import android.app.Activity;
import android.app.Application;
import android.app.NotificationManager;
import android.content.Context;
import android.content.Intent;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.core.app.NotificationCompat;

import com.igaworks.v2.core.AbxCommerce;
import com.igaworks.v2.core.AbxCommon;
import com.igaworks.v2.core.AbxGame;
import com.igaworks.v2.core.AdBrixRm;
import com.igaworks.v2.core.application.AbxActivityHelper;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.adbrix.sdk.component.AbxLog;
import io.adbrix.sdk.domain.function.Completion;
import io.adbrix.sdk.domain.model.ActionHistory;
import io.adbrix.sdk.domain.model.DfnInAppMessage;
import io.adbrix.sdk.domain.model.DfnInAppMessageFetchMode;
import io.adbrix.sdk.domain.model.Empty;
import io.adbrix.sdk.utils.CommonUtils;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import com.google.firebase.messaging.RemoteMessage;

/**
 * AdbrixrmflutterPlugin
 */
public class AdbrixrmFlutterPlugin implements FlutterPlugin, ActivityAware, MethodCallHandler {
    private static final String METHOD_CHANNEL = "com.adbrix.methodChannel";

    public static final String ERROR_CODE_SDK_ERROR = "-1";
    private Activity activity;
    private Context context;
    private MethodChannel methodChannel;

    public static void sdkInit(Application application, String appkey, String secretKey) {
        AbxActivityHelper.initializeSdk(application, appkey, secretKey);
    }

    public static void setRegistrationId(String token) {
        AdBrixRm.setRegistrationId(token);
    }

    public static void setPushEnable(boolean enable ) {
        AdBrixRm.setPushEnable(enable);
    }

    public static void setPushIconStyle(Context context, String smallIconName, String largeIconName, int argb) {
        AdBrixRm.setPushIconStyle(context, smallIconName, largeIconName, argb);
    }

    public static void setNotificationChannel(String channelName, String channelDescription) {
        AdBrixRm.createDefaultNotificationChannel(channelName, channelDescription);
    }

    public static void setNotificationOption(Context context, int priority, int visibility) {
        AdBrixRm.setNotificationOption(context, priority, visibility);
    }

    public static void deeplinkEventWithIntent(Intent intent) {
        AdBrixRm.deeplinkEventWithIntent(intent);
    }

    public static void setDeeplinkListener() {
        AdBrixRm.setDeeplinkListener(AdbrixrmEventChannel.shared());
    }

    public static void setDeferredDeeplinkListener() {
        AdBrixRm.setDeferredDeeplinkListener(AdbrixrmEventChannel.shared());
    }

    public static void setRemotePushMessageListener() {
        AdBrixRm.setRemotePushMessageListener(AdbrixrmEventChannel.shared());
    }

    @Deprecated
    public static void setLocalPushMessageListener() {
        AdBrixRm.setLocalPushMessageListener(AdbrixrmEventChannel.shared());
    }

    public static void setDfnInAppMessageAutoFetchListener() {
        AdBrixRm.setDfnInAppMessageAutoFetchListener(AdbrixrmEventChannel.shared());
    }

    public static void setLogListener() {
        AdBrixRm.setLogListener(AdbrixrmEventChannel.shared());
    }

    public static void setInAppMessageClickListener() {
        AdBrixRm.setInAppMessageClickListener(AdbrixrmEventChannel.shared());
    }

    public static void onMessageReceived(Context context, RemoteMessage remoteMessage) {
        AdBrixRm.onMessageReceived(context, remoteMessage);
    }

    // region Flutter Event
    private void initPlugin(Context context, BinaryMessenger messenger) {
        this.context = context;
        this.methodChannel = new MethodChannel(messenger, METHOD_CHANNEL);
        this.methodChannel.setMethodCallHandler(this);

        AdbrixrmEventChannel.shared().linkEventChannel(messenger);
        AdbrixrmEventChannel.shared().setStreamHandler();
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        initPlugin(flutterPluginBinding.getApplicationContext(), flutterPluginBinding.getBinaryMessenger());
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        methodChannel.setMethodCallHandler(null);
        AdbrixrmEventChannel.shared().unSetStreamHandler();
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        activity = binding.getActivity();
        methodChannel.setMethodCallHandler(this);
        AdbrixrmEventChannel.shared().setStreamHandler();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
        onDetachedFromActivity();
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        onAttachedToActivity(binding);
    }

    @Override
    public void onDetachedFromActivity() {
        this.activity = null;
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        String method = call.method;
        switch (method) {
            case "login":
                login(call);
                break;
            case "logout":
                logout();
                break;
            case "gdprForgetMe":
                gdprForgetMe();
                break;
            case "setAge":
                setAge(call);
                break;
            case "setGender":
                setGender(call);
                break;
            case "setUserProperties":
                saveUserProperties(call);
                break;
            case "clearUserProperties":
                clearUserProperties();
                break;
            case "event":
                events(call);
                break;
            case "flushAllEvents":
                flushAllEvents(result);
                break;
            case "setEventUploadCountInterval":
                setEventUploadCountInterval(call);
                break;
            case "setEventUploadTimeInterval":
                setEventUploadTimeInterval(call);
                break;
            case "setPushEnable":
                setPushEnable(call);
                break;
            case "setRegistrationId":
                setRegistrationId(call);
                break;
            case "setBigTextClientPushEvent":
                setBigTextClientPushEvent(call);
                break;
            case "setBigPictureClientPushEvent":
                setBigPictureClientPushEvent(call);
                break;
            case "cancelLocalPushNotification":
                cancelLocalPushNotification(call);
                break;
            case "getPushEventList":
                getPushEventList(result);
                break;
            case "setPushIconStyle":
                setPushIconStyle(call);
                break;
            case "setNotificationOption":
                setNotificationOption(call);
                break;
            case "setNotificationChannel":
                setNotificationChannel(call);
                break;
            case "setKakaoId":
                setKakaoId(call);
                break;
            case "setCiProperties":
                saveCiProperties(call);
                break;
            case "openPush":
                openPush(call);
                break;
            case "deleteUserDataAndStopSDK":
                deleteUserDataAndStopSDK(call, result);
                break;
            case "restartSDK":
                restartSDK(call, result);
                break;
            case "getSdkVersion":
                getSdkVersion(result);
                break;
            case "fetchActionHistoryByUserId":
                fetchActionHistoryByUserId(call, result);
                break;
            case "fetchActionHistoryByAdid":
                fetchActionHistoryByAdid(call, result);
                break;
            case "insertPushData":
                insertPushData(call);
                break;
            case "getActionHistory":
                getActionHistory(call, result);
                break;
            case "getAllActionHistory":
                getAllActionHistory(call, result);
                break;
            case "deleteActionHistory":
                deleteActionHistory(call, result);
                break;
            case "deleteAllActionHistoryByUserId":
                deleteAllActionHistoryByUserId(call, result);
                break;
            case "deleteAllActionHistoryByAdid":
                deleteAllActionHistoryByAdid(call, result);
                break;
            case "clearSyncedActionHistoryInLocalDB":
                clearSyncedActionHistoryInLocalDB(result);
                break;
            case "setInAppMessageFetchMode":
                setInAppMessageFetchMode(call);
                break;
            case "setInAppMessageToken":
                setInAppMessageToken(call);
                break;
            case "fetchInAppMessage":
                fetchInAppMessage(result);
                break;
            case "getAllInAppMessage":
                getAllInAppMessage(result);
                break;
            case "openInAppMessage":
                openInAppMessage(call, result);
                break;
            case "commonPurchase":
                commonPurchase(call);
                break;
            case "commonSignUp":
                commonSignUp(call);
                break;
            case "useCredit":
                commonUseCredit(call);
                break;
            case "appUpdate":
                commonAppUpdate(call);
                break;
            case "userInvite":
                commonInvite(call);
                break;
            case "commerceViewHome":
                commerceViewHome(call);
                break;
            case "commerceCategoryView":
                commerceCategoryView(call);
                break;
            case "commerceProductView":
                commerceProductView(call);
                break;
            case "commerceAddToCart":
                commerceAddToCart(call);
                break;
            case "commerceAddToWishList":
                commerceAddToWishList(call);
                break;
            case "commerceReviewOrder":
                commerceReviewOrder(call);
                break;
            case "commerceRefund":
                commerceRefund(call);
                break;
            case "commerceSearch":
                commerceSearch(call);
                break;
            case "commerceShare":
                commerceShare(call);
                break;
            case "commerceListView":
                commerceListView(call);
                break;
            case "commerceCartView":
                commerceCartView(call);
                break;
            case "commercePaymentInfoAdd":
                commercePaymentInfoAdd(call);
                break;
            case "gameTutorialComplete":
                gameTutorialComplete(call);
                break;
            case "gameLevelAchieved":
                gameLevelAchieved(call);
                break;
            case "gameCharacterCreated":
                gameCharacterCreated(call);
                break;
            case "gameStageCleared":
                gameStageCleared(call);
                break;
        }
    }
    // endregion
    // region AdBrixRm

    private void login(MethodCall call) {
        String userId = call.argument("userId");
        AdBrixRm.login(userId);
    }

    private void logout() {
        AdBrixRm.logout();
    }

    private void gdprForgetMe() {
        AdBrixRm.gdprForgetMe(context);
    }

    private void setAge(MethodCall call) {
        int age = call.arguments();
        AdBrixRm.setAge(age);
    }

    private void setGender(MethodCall call) {
        String gender = call.arguments();
        AdBrixRm.setGender(AdBrixRm.AbxGender.valueOf(gender));
    }

    private void saveUserProperties(MethodCall call) {
        HashMap<String, Object> properties = call.arguments();
        AdBrixRm.UserProperties userProperties = Utils.makeUserProperties(properties);
        AdBrixRm.saveUserProperties(userProperties);
    }

    @Deprecated
    private void clearUserProperties() {
        return;
    }

    private void events(MethodCall call) {
        String eventName = call.argument("eventName");
        if (call.hasArgument("attr")) {
            HashMap<String, Object> attr = call.argument("attr");
            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AdBrixRm.event(eventName, eventAttr);
                return;
            }
        }
        AdBrixRm.event(eventName);
    }

    private void flushAllEvents(final Result result) {
        AdBrixRm.flushAllEvents(new Completion<io.adbrix.sdk.domain.model.Result<Empty>>() {
            @Override
            public void handle(io.adbrix.sdk.domain.model.Result<Empty> emptyResult) {
                result.success(emptyResult.isSucceeded());
            }
        });
    }

    private void setEventUploadCountInterval(MethodCall call) {
        String argument = call.arguments();
        AdBrixRm.AdBrixEventUploadCountInterval interval;
        switch (argument) {
            case "MIN":
                interval = AdBrixRm.AdBrixEventUploadCountInterval.MIN;
                break;
            case "NORMAL":
                interval = AdBrixRm.AdBrixEventUploadCountInterval.NORMAL;
                break;
            case "MAX":
                interval = AdBrixRm.AdBrixEventUploadCountInterval.MAX;
                break;
            default:
                interval = AdBrixRm.AdBrixEventUploadCountInterval.NORMAL;
                break;
        }
        AdBrixRm.setEventUploadCountInterval(interval);
    }

    private void setEventUploadTimeInterval(MethodCall call) {
        String argument = call.arguments();
        AdBrixRm.AdBrixEventUploadTimeInterval interval;
        switch (argument) {
            case "MIN":
                interval = AdBrixRm.AdBrixEventUploadTimeInterval.MIN;
                break;
            case "NORMAL":
                interval = AdBrixRm.AdBrixEventUploadTimeInterval.NORMAL;
                break;
            case "MAX":
                interval = AdBrixRm.AdBrixEventUploadTimeInterval.MAX;
                break;
            default:
                interval = AdBrixRm.AdBrixEventUploadTimeInterval.NORMAL;
                break;
        }
        AdBrixRm.setEventUploadTimeInterval(interval);
    }

    private void setPushEnable(MethodCall call) {
        Boolean enable = call.arguments();
        AdBrixRm.setPushEnable(enable);
    }

    private void setRegistrationId(MethodCall call) {
        String value = call.arguments();
        AdBrixRm.setRegistrationId(value);
    }

    private void setBigTextClientPushEvent(MethodCall call) {
        String bigTextPushPropertiesJson = call.argument("bigTextPushPropertiesJson");
        Boolean alwaysIsShown = call.argument("alwaysIsShown");
        if (CommonUtils.isNullOrEmpty(bigTextPushPropertiesJson)) {
            AbxLog.e("setBigTextClientPushEvent() bigTextPushPropertiesJson is empty", false);
            return;
        }
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject = new JSONObject(bigTextPushPropertiesJson);
        } catch (JSONException e) {
            AbxLog.e("setBigTextClientPushEvent() parse error: ", e, false);
        }
        AdBrixRm.setBigTextClientPushEvent(activity.getApplicationContext(),
                AdBrixRm.BigTextPushProperties.fromJSONObject(jsonObject), alwaysIsShown);
    }

    private void setBigPictureClientPushEvent(MethodCall call) {
        String bigPicturePushPropertiesJson = call.argument("bigPicturePushPropertiesJson");
        Boolean alwaysIsShown = call.argument("alwaysIsShown");
        if (CommonUtils.isNullOrEmpty(bigPicturePushPropertiesJson)) {
            AbxLog.e("setBigPictureClientPushEvent() bigPicturePushPropertiesJson is empty", false);
            return;
        }
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject = new JSONObject(bigPicturePushPropertiesJson);
        } catch (JSONException e) {
            AbxLog.e("setBigPictureClientPushEvent() parse error: ", e, false);
        }
        AdBrixRm.setBigPictureClientPushEvent(activity.getApplicationContext(),
                AdBrixRm.BigPicturePushProperties.fromJSONObject(jsonObject), alwaysIsShown);
    }

    private void cancelLocalPushNotification(MethodCall call) {
        ArrayList<String> list = call.argument("eventId");
        if (list == null) {
            AbxLog.e("cancelLocalPushNotification eventIds is null", false);
            return;
        }
        int[] eventIdArray = new int[list.size()];
        for (int i = 0; i < list.size(); i++) {
            int temp = 0;
            try {
                temp = Integer.parseInt(list.get(i));
            } catch (Exception e) {
                AbxLog.w(e, false);
            }
            eventIdArray[i] = temp;
        }
        AdBrixRm.cancelLocalPushNotification(context, eventIdArray);
    }

    private void getPushEventList(Result result) {
        JSONArray pushEventList = AdBrixRm.getPushEventList();
        result.success(pushEventList.toString());
    }

    private void setPushIconStyle(MethodCall call) {
        String smallIconName = call.argument("smallIconName");
        String largeIconName = call.argument("largeIconName");
        if (call.hasArgument("argb")) {
            Integer argb = call.argument("argb");
            AdBrixRm.setPushIconStyle(activity.getApplicationContext(), smallIconName, largeIconName, argb);
            return;
        }
        AdBrixRm.setPushIconStyle(activity.getApplicationContext(), smallIconName, largeIconName);
    }

    private void setNotificationOption(MethodCall call) {
        Integer priority = call.argument("priority");
        Integer visibility = call.argument("visibility");
        AdBrixRm.setNotificationOption(activity.getApplicationContext(), priority, visibility);
    }

    private void setNotificationChannel(MethodCall call) {
        String channelName = call.argument("channelName");
        String channelDescription = call.argument("channelDescription");
        AdBrixRm.createDefaultNotificationChannel(channelName, channelDescription);
    }

    private void setKakaoId(MethodCall call) {
        String kakaoId = call.arguments();
        AdBrixRm.setKakaoId(kakaoId);
    }

    private void saveCiProperties(MethodCall call) {
        HashMap<String, Object> properties = call.arguments();
        AdBrixRm.CiProperties ciProperties = Utils.makeCiProperties(properties);
        AdBrixRm.saveCiProperties(ciProperties);
    }

    private void openPush(MethodCall call) {
        HashMap<String, Object> pushJson = call.argument("pushJson");
        if (CommonUtils.isNullOrEmpty(pushJson)) {
            AbxLog.e("openPush() pushJson is empty", false);
            return;
        }
        try {
            JSONObject simplePushModel = new JSONObject(pushJson);
            AdBrixRm.AbxRemotePushModel abxRemotePushModel = new AdBrixRm.AbxRemotePushModel(
                    simplePushModel.optString("abx:gf:campaign_id", ""),
                    simplePushModel.optInt("abx:gf:campaign_revision_no", 0),
                    simplePushModel.optString("abx:gf:step_id", ""),
                    simplePushModel.optString("abx:gf:cycle_time", ""));
            AdBrixRm.openPush(abxRemotePushModel);
        } catch (Exception e) {
            Log.d("abxrm", "AbxUnityActivity::openPush - Adbrix push tracking parameters don't exist!");
        }
    }

    private void deleteUserDataAndStopSDK(MethodCall call, final Result result) {
        String userId = call.arguments();
        Runnable onSuccessRunnable = new Runnable() {
            @Override
            public void run() {
                result.success(true);
            }
        };
        Runnable onFailRunnable = new Runnable() {
            @Override
            public void run() {
                result.success(false);
            }
        };
        AdBrixRm.deleteUserDataAndStopSDK(userId, onSuccessRunnable, onFailRunnable);
    }

    private void restartSDK(MethodCall call, final Result result) {
        String userId = call.arguments();
        Runnable onSuccessRunnable = new Runnable() {
            @Override
            public void run() {
                result.success(true);
            }
        };
        Runnable onFailRunnable = new Runnable() {
            @Override
            public void run() {
                result.success(false);
            }
        };
        AdBrixRm.restartSDK(userId, onSuccessRunnable, onFailRunnable);
    }

    private void getSdkVersion(Result result) {
        String sdkVersion = AdBrixRm.SDKVersion();
        result.success(sdkVersion);
    }

    private void fetchActionHistoryByUserId(MethodCall call, final Result result) {
        String token = call.argument("token");
        ArrayList<String> actionType = call.argument("actionType");
        if (CommonUtils.isNullOrEmpty(actionType)) {
            AbxLog.e("fetchActionHistoryByUserId() actionType is null or empty", false);
            return;
        }
        AdBrixRm.fetchActionHistoryByUserId(token, actionType,
                new Completion<io.adbrix.sdk.domain.model.Result<List<ActionHistory>>>() {
                    @Override
                    public void handle(io.adbrix.sdk.domain.model.Result<List<ActionHistory>> listResult) {
                        List<ActionHistory> list = listResult.getOrDefault(new ArrayList<ActionHistory>());
                        JSONArray array = new JSONArray();
                        for (ActionHistory index : list) {
                            if (index == null) {
                                continue;
                            }
                            try {
                                array.put(index);
                            } catch (Exception e) {
                                AbxLog.e("fetchActionHistoryByUserId() ", e, false);
                            }
                        }
                        if (listResult.isSucceeded()) {
                            result.success(array.toString());
                        } else {
                            result.error(ERROR_CODE_SDK_ERROR, listResult.toString(), null);
                        }
                    }
                });
    }

    private void fetchActionHistoryByAdid(MethodCall call, final Result result) {
        String token = call.argument("token");
        ArrayList<String> actionType = call.argument("actionType");
        if (CommonUtils.isNullOrEmpty(actionType)) {
            AbxLog.e("fetchActionHistoryByAdid() actionType is null or empty", false);
            return;
        }
        AdBrixRm.fetchActionHistoryByAdid(token, actionType,
                new Completion<io.adbrix.sdk.domain.model.Result<List<ActionHistory>>>() {
                    @Override
                    public void handle(io.adbrix.sdk.domain.model.Result<List<ActionHistory>> listResult) {
                        List<ActionHistory> list = listResult.getOrDefault(new ArrayList<ActionHistory>());
                        JSONArray array = new JSONArray();
                        for (ActionHistory index : list) {
                            if (index == null) {
                                continue;
                            }
                            try {
                                array.put(index);
                            } catch (Exception e) {
                                AbxLog.e("fetchActionHistoryByAdid() ", e, false);
                            }
                        }
                        if (listResult.isSucceeded()) {
                            result.success(array.toString());
                        } else {
                            result.error(ERROR_CODE_SDK_ERROR, listResult.toString(), null);
                        }
                    }
                });
    }

    private void insertPushData(MethodCall call) {
        Map<String, String> map = call.arguments();
        AdBrixRm.insertPushData(map);
    }

    private void getActionHistory(MethodCall call, final Result result) {
        Integer skip = call.argument("skip");
        Integer limit = call.argument("limit");
        ArrayList<String> actionType = call.argument("actionType");
        AdBrixRm.getActionHistory(skip, limit, actionType,
                new Completion<io.adbrix.sdk.domain.model.Result<List<ActionHistory>>>() {
                    @Override
                    public void handle(io.adbrix.sdk.domain.model.Result<List<ActionHistory>> listResult) {
                        List<ActionHistory> list = listResult.getOrDefault(new ArrayList<ActionHistory>());
                        JSONArray array = new JSONArray();
                        for (ActionHistory index : list) {
                            if (index == null) {
                                continue;
                            }
                            try {
                                array.put(index);
                            } catch (Exception e) {
                                AbxLog.e("getActionHistory() ", e, false);
                            }
                        }
                        if (listResult.isSucceeded()) {
                            result.success(array.toString());
                        } else {
                            result.error(ERROR_CODE_SDK_ERROR, listResult.toString(), null);
                        }
                    }
                });
    }

    private void getAllActionHistory(MethodCall call, final Result result) {
        ArrayList<String> actionType = call.argument("actionType");
        AdBrixRm.getAllActionHistory(actionType,
                new Completion<io.adbrix.sdk.domain.model.Result<List<ActionHistory>>>() {
                    @Override
                    public void handle(io.adbrix.sdk.domain.model.Result<List<ActionHistory>> listResult) {
                        List<ActionHistory> list = listResult.getOrDefault(new ArrayList<ActionHistory>());
                        JSONArray array = new JSONArray();
                        for (ActionHistory index : list) {
                            if (index == null) {
                                continue;
                            }
                            try {
                                array.put(index);
                            } catch (Exception e) {
                                AbxLog.e("getAllActionHistory() ", e, false);
                            }
                        }
                        if (listResult.isSucceeded()) {
                            result.success(array.toString());
                        } else {
                            result.error(ERROR_CODE_SDK_ERROR, listResult.toString(), null);
                        }
                    }
                });
    }

    private void deleteActionHistory(MethodCall call, final Result result) {
        String token = call.argument("token");
        String historyId = call.argument("historyId");
        Long timestamp = call.argument("timestamp");
        AdBrixRm.deleteActionHistory(token, historyId, timestamp,
                new Completion<io.adbrix.sdk.domain.model.Result<Empty>>() {
                    @Override
                    public void handle(io.adbrix.sdk.domain.model.Result<Empty> emptyResult) {
                        result.success(emptyResult.isSucceeded());
                    }
                });
    }

    private void deleteAllActionHistoryByUserId(MethodCall call, final Result result) {
        String token = call.argument("token");
        AdBrixRm.deleteAllActionHistoryByUserId(token, new Completion<io.adbrix.sdk.domain.model.Result<Empty>>() {
            @Override
            public void handle(io.adbrix.sdk.domain.model.Result<Empty> emptyResult) {
                result.success(emptyResult.isSucceeded());
            }
        });
    }

    private void deleteAllActionHistoryByAdid(MethodCall call, final Result result) {
        String token = call.argument("token");
        AdBrixRm.deleteAllActionHistoryByAdid(token, new Completion<io.adbrix.sdk.domain.model.Result<Empty>>() {
            @Override
            public void handle(io.adbrix.sdk.domain.model.Result<Empty> emptyResult) {
                result.success(emptyResult.isSucceeded());
            }
        });
    }

    private void clearSyncedActionHistoryInLocalDB(final Result result) {
        AdBrixRm.clearSyncedActionHistoryInLocalDB(new Completion<io.adbrix.sdk.domain.model.Result<Empty>>() {
            @Override
            public void handle(io.adbrix.sdk.domain.model.Result<Empty> emptyResult) {
                result.success(emptyResult.isSucceeded());
            }
        });
    }

    private void setInAppMessageFetchMode(MethodCall call) {
        Integer mode = call.argument("mode");
        DfnInAppMessageFetchMode dfnInAppMessageFetchMode;
        switch (mode) {
            case 0: {
                dfnInAppMessageFetchMode = DfnInAppMessageFetchMode.USER_ID;
                break;
            }
            case 1: {
                dfnInAppMessageFetchMode = DfnInAppMessageFetchMode.ADID;
                break;
            }
            default: {
                dfnInAppMessageFetchMode = DfnInAppMessageFetchMode.USER_ID;
                AbxLog.e("setInAppMessageFetchMode() unknown mode!", false);
                break;
            }
        }
        AdBrixRm.setInAppMessageFetchMode(dfnInAppMessageFetchMode);
    }

    private void setInAppMessageToken(MethodCall call) {
        String token = call.argument("token");
        AdBrixRm.setInAppMessageToken(token);
    }

    private void fetchInAppMessage(final Result result) {
        AdBrixRm.fetchInAppMessage(new Completion<io.adbrix.sdk.domain.model.Result<Empty>>() {
            @Override
            public void handle(io.adbrix.sdk.domain.model.Result<Empty> emptyResult) {
                result.success(emptyResult.isSucceeded());
            }
        });
    }

    private void getAllInAppMessage(final Result result) {
        AdBrixRm.getAllInAppMessage(new Completion<io.adbrix.sdk.domain.model.Result<List<DfnInAppMessage>>>() {
            @Override
            public void handle(io.adbrix.sdk.domain.model.Result<List<DfnInAppMessage>> listResult) {
                List<DfnInAppMessage> list = listResult.getOrDefault(new ArrayList<DfnInAppMessage>());
                JSONArray array = new JSONArray();
                for (DfnInAppMessage index : list) {
                    if (index == null) {
                        continue;
                    }
                    try {
                        array.put(index);
                    } catch (Exception e) {
                        AbxLog.e("getAllInAppMessage() error", e, false);
                    }
                }
                if (listResult.isSucceeded()) {
                    result.success(array.toString());
                } else {
                    result.error(ERROR_CODE_SDK_ERROR, listResult.toString(), null);
                }
            }
        });
    }

    private void openInAppMessage(MethodCall call, final Result result) {
        String campaignId = call.argument("campaignId");
        AdBrixRm.openInAppMessage(campaignId, new Completion<io.adbrix.sdk.domain.model.Result<Empty>>() {
            @Override
            public void handle(io.adbrix.sdk.domain.model.Result<Empty> emptyResult) {
                result.success(emptyResult.isSucceeded());
            }
        });
    }
    // endregion

    // region AbxCommon

    private void commonPurchase(MethodCall call) {

        String orderId = call.argument("orderId");
        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        Double discount = call.argument("discount");
        Double orderSale = call.argument("orderSale");
        Double deliveryCharge = call.argument("deliveryCharge");
        String paymentMethod = call.argument("paymentMethod");
        HashMap<String, Object> attr = call.argument("attr");

        if (productList != null && orderId != null && orderSale != null && discount != null && deliveryCharge != null
                && paymentMethod != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);

            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AdBrixRm.CommonProperties.Purchase purchaseAttr = new AdBrixRm.CommonProperties.Purchase()
                        .setAttrModel(evetAttr);
                AbxCommon.purchase(orderId, productModelList, orderSale, discount, deliveryCharge,
                        AdBrixRm.CommercePaymentMethod.getMethodByMethodCode(paymentMethod), purchaseAttr);
            } else {
                AbxCommon.purchase(orderId, productModelList, orderSale, discount, deliveryCharge,
                        AdBrixRm.CommercePaymentMethod.getMethodByMethodCode(paymentMethod));
            }
        }
    }

    private void commonSignUp(MethodCall call) {

        Integer channel = call.argument("channel");
        HashMap<String, Object> attr = call.argument("attr");

        if (channel != null) {

            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AdBrixRm.CommonProperties.SignUp signUpUserInfo = new AdBrixRm.CommonProperties.SignUp()
                        .setAttrModel(eventAttr);
                AbxCommon.signUp(AdBrixRm.CommonSignUpChannel.getChannelByChannelCode(channel), signUpUserInfo);

            } else {
                AbxCommon.signUp(AdBrixRm.CommonSignUpChannel.getChannelByChannelCode(channel));

            }
        }
    }

    private void commonUseCredit(MethodCall call) {

        HashMap<String, Object> attr = call.argument("attr");

        if (attr != null) {
            AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
            AdBrixRm.CommonProperties.UseCredit useCreditInfo = new AdBrixRm.CommonProperties.UseCredit()
                    .setAttrModel(eventAttr);
            AbxCommon.useCredit(useCreditInfo);

        } else {
            AbxCommon.useCredit();

        }
    }

    private void commonAppUpdate(MethodCall call) {

        String preVersion = call.argument("preVersion");
        String currVersion = call.argument("currVersion");
        HashMap<String, Object> attr = call.argument("attr");

        if (attr != null) {
            AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
            AdBrixRm.CommonProperties.AppUpdate updateInfo = new AdBrixRm.CommonProperties.AppUpdate()
                    .setPrevVersion(preVersion)
                    .setCurrVersion(currVersion)
                    .setAttrModel(eventAttr);
            AbxCommon.appUpdate(updateInfo);

        } else {
            AdBrixRm.CommonProperties.AppUpdate updateInfo = new AdBrixRm.CommonProperties.AppUpdate()
                    .setPrevVersion(preVersion)
                    .setCurrVersion(currVersion);
            AbxCommon.appUpdate(updateInfo);

        }
    }

    private void commonInvite(MethodCall call) {

        Integer inviteChannel = call.argument("inviteChannel");
        HashMap<String, Object> attr = call.argument("attr");

        if (inviteChannel != null) {

            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AdBrixRm.CommonProperties.Invite inviteInfo = new AdBrixRm.CommonProperties.Invite()
                        .setAttrModel(eventAttr);
                AbxCommon.invite(AdBrixRm.CommonInviteChannel.getChannelByChannelCode(inviteChannel), inviteInfo);

            } else {
                AbxCommon.invite(AdBrixRm.CommonInviteChannel.getChannelByChannelCode(inviteChannel));

            }
        }
    }
    // endregion

    // region AbxCommerce
    private void commerceViewHome(MethodCall call) {
        HashMap<String, Object> attr = call.argument("attr");
        if (attr != null) {
            AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
            AbxCommerce.viewHome(eventAttr);
        } else {
            AbxCommerce.viewHome();
        }
    }

    private void commerceCategoryView(MethodCall call) {
        ArrayList<String> categoryModel = call.argument("categoryModel");
        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        HashMap<String, Object> attr = call.argument("attr");
        if (productList != null && categoryModel != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);
            AdBrixRm.CommerceCategoriesModel categoriesModel = new AdBrixRm.CommerceCategoriesModel();
            for (int i = 0, n = categoryModel.size(); i < n; i++) {
                String categoryListName = categoryModel.get(i);
                categoriesModel.setCategory(categoryListName);
            }
            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AbxCommerce.categoryView(categoriesModel, productModelList, evetAttr);
            } else {
                AbxCommerce.categoryView(categoriesModel, productModelList);
            }
        }
    }

    private void commerceProductView(MethodCall call) {

        HashMap<String, Object> productModel = call.argument("productModel");
        HashMap<String, Object> attr = call.argument("attr");

        if (productModel != null) {
            AdBrixRm.CommerceProductModel userProduct = Utils.makeProductModel(productModel);

            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AbxCommerce.productView(userProduct, eventAttr);

            } else {
                AbxCommerce.productView(userProduct);
            }

        }

    }

    private void commerceAddToCart(MethodCall call) {

        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        HashMap<String, Object> attr = call.argument("attr");

        if (productList != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);

            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AbxCommerce.addToCart(productModelList, evetAttr);

            } else {
                AbxCommerce.addToCart(productModelList);

            }
        }
    }

    private void commerceAddToWishList(MethodCall call) {

        HashMap<String, Object> productModel = call.argument("productModel");
        HashMap<String, Object> attr = call.argument("attr");

        if (productModel != null) {
            AdBrixRm.CommerceProductModel userProduct = Utils.makeProductModel(productModel);

            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AbxCommerce.addToWishList(userProduct, eventAttr);

            } else {
                AbxCommerce.addToWishList(userProduct);
            }
        }
    }

    private void commerceReviewOrder(MethodCall call) {

        String orderId = call.argument("orderId");
        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        Double discount = call.argument("discount");
        Double deliveryCharge = call.argument("deliveryCharge");
        HashMap<String, Object> attr = call.argument("attr");

        if (productList != null && orderId != null && discount != null && deliveryCharge != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);

            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AbxCommerce.reviewOrder(orderId, productModelList, discount, deliveryCharge, evetAttr);

            } else {
                AbxCommerce.reviewOrder(orderId, productModelList, discount, deliveryCharge);

            }
        }
    }

    private void commerceRefund(MethodCall call) {

        String orderId = call.argument("orderId");
        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        Double penaltyCharge = call.argument("penaltyCharge");
        HashMap<String, Object> attr = call.argument("attr");

        if (productList != null && orderId != null && penaltyCharge != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);

            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AbxCommerce.refund(orderId, productModelList, penaltyCharge, evetAttr);

            } else {
                AbxCommerce.refund(orderId, productModelList, penaltyCharge);

            }
        }
    }

    private void commerceSearch(MethodCall call) {

        String keyword = call.argument("keyword");
        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        HashMap<String, Object> attr = call.argument("attr");

        if (productList != null && keyword != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);

            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AbxCommerce.search(keyword, productModelList, evetAttr);

            } else {
                AbxCommerce.search(keyword, productModelList);

            }
        }
    }

    private void commerceShare(MethodCall call) {

        HashMap<String, Object> productModel = call.argument("productModel");
        HashMap<String, Object> attr = call.argument("attr");
        String sharingChannel = call.argument("sharingChannel");

        if (productModel != null && sharingChannel != null) {
            AdBrixRm.CommerceProductModel userProduct = Utils.makeProductModel(productModel);
            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AbxCommerce.share(AdBrixRm.CommerceSharingChannel.getChannelByChannelCode(sharingChannel), userProduct,
                        eventAttr);
            } else {
                AbxCommerce.share(AdBrixRm.CommerceSharingChannel.getChannelByChannelCode(sharingChannel), userProduct);
            }
        }
    }

    private void commerceListView(MethodCall call) {

        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        HashMap<String, Object> attr = call.argument("attr");

        if (productList != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);

            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AbxCommerce.listView(productModelList, evetAttr);

            } else {
                AbxCommerce.listView(productModelList);

            }
        }
    }

    private void commerceCartView(MethodCall call) {

        ArrayList<HashMap<String, Object>> productList = call.argument("productList");
        HashMap<String, Object> attr = call.argument("attr");

        if (productList != null) {
            ArrayList<AdBrixRm.CommerceProductModel> productModelList = Utils.makeProductList(productList);

            if (attr != null) {
                AdBrixRm.AttrModel evetAttr = Utils.makeAttrModel(attr);
                AbxCommerce.cartView(productModelList, evetAttr);

            } else {
                AbxCommerce.cartView(productModelList);

            }
        }
    }

    private void commercePaymentInfoAdd(MethodCall call) {
        HashMap<String, Object> attr = call.argument("attr");
        if (attr != null) {
            AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
            AbxCommerce.paymentInfoAdded(eventAttr);
        } else {
            AbxCommerce.paymentInfoAdded();
        }
    }
    // endregion

    // region AbxGame
    private void gameTutorialComplete(MethodCall call) {

        Boolean isSkip = call.argument("isSkip");
        HashMap<String, Object> attr = call.argument("attr");

        if (isSkip != null) {

            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AdBrixRm.GameProperties.TutorialComplete gameProperties = new AdBrixRm.GameProperties.TutorialComplete()
                        .setIsSkip(isSkip)
                        .setAttrModel(eventAttr);
                AbxGame.tutorialComplete(gameProperties);

            } else {
                AdBrixRm.GameProperties.TutorialComplete gameProperties = new AdBrixRm.GameProperties.TutorialComplete()
                        .setIsSkip(isSkip);
                AbxGame.tutorialComplete(gameProperties);

            }

        }

    }

    private void gameLevelAchieved(MethodCall call) {
        Integer levelAchieved = call.argument("levelAchieved");
        HashMap<String, Object> attr = call.argument("attr");
        if (levelAchieved != null) {
            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AdBrixRm.GameProperties.LevelAchieved gameProperties = new AdBrixRm.GameProperties.LevelAchieved()
                        .setLevel(levelAchieved)
                        .setAttrModel(eventAttr);
                AbxGame.levelAchieved(gameProperties);
            } else {
                AdBrixRm.GameProperties.LevelAchieved gameProperties = new AdBrixRm.GameProperties.LevelAchieved()
                        .setLevel(levelAchieved);
                AbxGame.levelAchieved(gameProperties);
            }
        }
    }

    private void gameCharacterCreated(MethodCall call) {

        HashMap<String, Object> attr = call.argument("attr");

        if (attr != null) {
            AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
            AdBrixRm.GameProperties.CharacterCreated gameProperties = new AdBrixRm.GameProperties.CharacterCreated()
                    .setAttrModel(eventAttr);
            AbxGame.characterCreated(gameProperties);

        } else {
            AbxGame.characterCreated();

        }
    }

    private void gameStageCleared(MethodCall call) {

        String stageName = call.argument("stageName");
        HashMap<String, Object> attr = call.argument("attr");

        if (stageName != null) {

            if (attr != null) {
                AdBrixRm.AttrModel eventAttr = Utils.makeAttrModel(attr);
                AdBrixRm.GameProperties.StageCleared gameProperties = new AdBrixRm.GameProperties.StageCleared()
                        .setStageName(stageName)
                        .setAttrModel(eventAttr);
                AbxGame.stageCleared(gameProperties);

            } else {
                AdBrixRm.GameProperties.StageCleared gameProperties = new AdBrixRm.GameProperties.StageCleared()
                        .setStageName(stageName);
                AbxGame.stageCleared(gameProperties);

            }

        }

    }

    // endregion
}
