//
//  AdBrixUtility.swift
//  adbrixrm_flutter
//
//  Created by Jin-uk Park on 2020/10/28.
//

import Foundation
import AdBrixRmKit

public class AdBrixUtility {
    
    public static func AdBrixRmAttrModelMakeAttrModel(attr: [String: Any]) -> AdBrixRmAttrModel {
        
        let attrModel = AdBrixRmAttrModel()
        for (key, value) in attr {
            switch value {
            case is String:
                attrModel.setAttrDataString(key, value as! String)
            case is Int:
                attrModel.setAttrDataInt(key, value as! Int)
            case is Bool:
                attrModel.setAttrDataBool(key, value as! Bool)
            case is Double:
                attrModel.setAttrDataDouble(key, value as! Double)
            default:
                continue
            }
        }
        return attrModel
        
    }

    
    public static func AdBrixRmMakeProductModel (productModel : [String: Any]) -> AdBrixRmCommerceProductModel {
        
        let adBrix = AdBrixRM.getInstance
        var productId : String = "productId"
        var productName : String = "productName"
        var price : Double = 1.00
        var discount : Double = 1.00
        var quantity : Int = 1
        var currency : String = "krw"
        var category : Array<String>? = nil
        var productAttr : [String: Any]? = nil
        var currencyName : String = "krw"
        
        for (key, value) in productModel {
            
            if key == "productId" {
                productId = value as? String ?? "productId"
                
            } else if key == "productName" {
                productName = value as? String ?? "productName"
                
            } else if key == "price" {
                price = value as? Double ?? 1.0
                
            } else if key == "discount" {
                discount = value as? Double ?? 1.0
                
            } else if key == "quantity" {
                quantity = value as? Int ?? 1
                
            } else if key == "currency" {
                currency = value as? String ?? "KR_KRW"
                
                switch currency {
                case "KR_KRW":
                    currencyName = adBrix.getCurrencyString(AdBrixRM.AdBrixRmCurrencyType.KRW.rawValue)
                case "US_USD":
                    currencyName = adBrix.getCurrencyString(AdBrixRM.AdBrixRmCurrencyType.USD.rawValue)
                case "JP_JPY":
                    currencyName = adBrix.getCurrencyString(AdBrixRM.AdBrixRmCurrencyType.JPY.rawValue)
                case "EU_EUR":
                    currencyName = adBrix.getCurrencyString(AdBrixRM.AdBrixRmCurrencyType.EUR.rawValue)
                case "UK_GBP":
                    currencyName = adBrix.getCurrencyString(AdBrixRM.AdBrixRmCurrencyType.GBP.rawValue)
                case "CH_CNY":
                    currencyName = adBrix.getCurrencyString(AdBrixRM.AdBrixRmCurrencyType.CNY.rawValue)
                case "TW_TWD":
                    currencyName = adBrix.getCurrencyString(AdBrixRM.AdBrixRmCurrencyType.TWD.rawValue)
                case "HK_HKD":
                    currencyName = adBrix.getCurrencyString(AdBrixRM.AdBrixRmCurrencyType.HKD.rawValue)
                case "ID_IDR":
                    currencyName = adBrix.getCurrencyString(AdBrixRM.AdBrixRmCurrencyType.IDR.rawValue)
                case "IN_INR":
                    currencyName = adBrix.getCurrencyString(AdBrixRM.AdBrixRmCurrencyType.INR.rawValue)
                case "RU_RUB":
                    currencyName = adBrix.getCurrencyString(AdBrixRM.AdBrixRmCurrencyType.RUB.rawValue)
                case "TH_THB":
                    currencyName = adBrix.getCurrencyString(AdBrixRM.AdBrixRmCurrencyType.THB.rawValue)
                case "VN_VND":
                    currencyName = adBrix.getCurrencyString(AdBrixRM.AdBrixRmCurrencyType.VND.rawValue)
                case "MY_MYR":
                    currencyName = adBrix.getCurrencyString(AdBrixRM.AdBrixRmCurrencyType.MYR.rawValue)
                default:
                    print("There is something wrong on building currencyUnit")
                    
                }
                
            } else if key == "category" {
                category = value as? [String]
                
            } else if key == "productAttr" {
                productAttr = value as? [String: Any]
                
            } else {
                print("There is something wrong on building productModel")
                
            }
        }
        
        if productAttr != nil {
            
            return adBrix.createCommerceProductDataWithAttr(productId: productId, productName: productName, price: price, quantity: quantity, discount: discount, currencyString: currencyName, category: AdBrixUtility.AdBrixRmMakeProductCategory(categoryList: category!), productAttrsMap: AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: productAttr!))
                        
        } else {
            
            return adBrix.createCommerceProductDataWithAttr(productId: productId, productName: productName, price: price, quantity: quantity, discount: discount, currencyString: currencyName, category: AdBrixUtility.AdBrixRmMakeProductCategory(categoryList: category!), productAttrsMap: nil)
            
        }
        
    }
    
    public static func AdBrixRmMakeProductList(productList : [[String: Any]]) -> [AdBrixRmCommerceProductModel] {
        
        var arr = [AdBrixRmCommerceProductModel]()
        for product in productList{
            
            let productModel = AdBrixRmMakeProductModel(productModel: product)
            
            arr.append(productModel)
            
        }
        
        return arr
        
    }
    
    public static func AdBrixRmMakeProductCategory (categoryList : [String]) -> AdBrixRmCommerceProductCategoryModel {
        
        let adBrix = AdBrixRM.getInstance
        var categoryResult = AdBrixRmCommerceProductCategoryModel()
        
        let categoryCount = categoryList.count
        
        if categoryCount == 1 {
            
            categoryResult = adBrix.createCommerceProductCategoryData(category: categoryList[0])
            
        } else if categoryCount == 2 {
            
            categoryResult = adBrix.createCommerceProductCategoryData(category: categoryList[0], category2: categoryList[1])
            
        } else if categoryCount == 3 {
            
            categoryResult = adBrix.createCommerceProductCategoryData(category: categoryList[0], category2: categoryList[1], category3: categoryList[2])

        } else if categoryCount == 4 {
            
            categoryResult = adBrix.createCommerceProductCategoryData(category: categoryList[0], category2: categoryList[1], category3: categoryList[2], category4: categoryList[3])

        } else {
            
            categoryResult = adBrix.createCommerceProductCategoryData(category: categoryList[0], category2: categoryList[1], category3: categoryList[2], category4: categoryList[3], category5: categoryList[4])

        }
        
        return categoryResult
        
    }
    
}

extension Dictionary where Value: Any {
    func toJsonFormat() -> String? {
        do {
            let jsonData = try JSONSerialization.data(withJSONObject: self, options: .prettyPrinted)
            
            if jsonData.isEmpty == false {
                if let jsonString = String(data: jsonData, encoding: .utf8) {
                    return jsonString
                }
                else {
                    return nil
                }
            }
            else {
                return nil
            }
        }
        catch {
            return nil
        }
    }
}
