import Flutter
import UIKit
import AdBrixRmKit

@objc
public class AdbrixrmFlutterPlugin: NSObject, FlutterPlugin {
    
    static let METHOD_CHANNEL = "com.adbrix.methodChannel"
    
    static let EVENT_CHANNEL_Deeplink = "com.adbrix.eventChannel.deeplink"
    static let EVENT_CHANNEL_DeferredDeeplink = "com.adbrix.eventChannel.deferredDeeplink"
    static let EVENT_CHANNEL_PushLocalCallback = "com.adbrix.eventChannel.pushLocalCallBack"
    static let EVENT_CHANNEL_PushRemoteCallback = "com.adbrix.eventChannel.pushRemoteCallback"
    static let EVENT_CHANNEL_OnReceiveInAppMessageClick = "com.adbrix.eventChannel.onReceiveInAppMessageClick"
    static let EVENT_CHANNEL_DidFetchInAppMessage = "com.adbrix.eventChannel.didFetchInAppMessage"
    static let EVENT_CHANNEL_DidPrintLog = "com.adbrix.eventChannel.didPrintLog"
    
    let ERROR_CODE_SDK_ERROR = "-1";
    
    let adBrix = AdBrixRM.getInstance
    
    @objc
    public static func sdkInit(appKey: String, secretKey: String) {
        AdBrixRM.getInstance.initAdBrix(appKey: appKey, secretKey: secretKey)
    }
    
    ///
    /// You should call setDeeplinkListener() first if you want to set deeplink Listener for flutter
    @objc
    public static func deeplinkOpen(with url: URL) {
        AdBrixRM.getInstance.deepLinkOpen(url: url)
    }

    @objc
    public static func setDeeplinkListener() {
        AdBrixRM.getInstance.setDeeplinkDelegate(delegate: AdBrixEventChannel.shared)
    }

    
    @objc
    public static func setDeferredDeeplinkListener() {
        AdBrixRM.getInstance.setDeferredDeeplinkDelegate(delegate: AdBrixEventChannel.shared)
    }
    
    @available(*, deprecated)
    @objc
    public static func localPushOpen(center: UNUserNotificationCenter, response: UNNotificationResponse) {
        AdBrixRM.getInstance.setAdBrixRmPushLocalDelegate(delegate: AdBrixEventChannel.shared)
        AdBrixRM.getInstance.userNotificationCenter(center: center, response: response)
    }

    @objc
    public static func setRemotePushMessageListener() {
        AdBrixRM.getInstance.setAdBrixRmPushRemoteDelegate(delegate: AdBrixEventChannel.shared)
    }
    
    @objc
    public static func remotePushOpen(center: UNUserNotificationCenter, response: UNNotificationResponse) {
        AdBrixRM.getInstance.userNotificationCenter(center: center, response: response)
    }

    @objc
    public static func willPresentRemotePush(notification: UNNotification, completionHandler: @escaping (UNNotificationPresentationOptions) -> ()) {
        if let apsInfo = notification.request.content.userInfo["aps"] as? [String : Any] {
            if let genWhileRun = apsInfo["gen_while_run"] as? String,
                genWhileRun == "1" {
                completionHandler([.alert, .sound])
            }
        }
        completionHandler([])

    }
    
    @objc
    public static func setInAppMessageClickListener() {
        AdBrixRM.getInstance.setInAppMessageClickDelegate(delegate: AdBrixEventChannel.shared)
    }
    
    @objc
    public static func setDidFetchInAppMessageListener() {
        AdBrixRM.getInstance.setInAppMessageAutoFetchDelegate(delegate: AdBrixEventChannel.shared)
    }
    
    @objc
    public static func startGettingIDFA () {
        AdBrixRM.getInstance.startGettingIDFA();
    }
    
    @objc
    public static func stopGettingIDFA () {
        AdBrixRM.getInstance.stopGettingIDFA();
    }
    
    @objc
    public static func setPushEnable(toPushEnable: Bool) {
        AdBrixRM.getInstance.setPushEnable(toPushEnable: toPushEnable)
    }
    
    @objc
    public static func setRegistrationId(deviceToken: Data) {
        AdBrixRM.getInstance.setRegistrationId(deviceToken: deviceToken)
    }
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let instance = AdbrixrmFlutterPlugin()
        
        let methodChannel = FlutterMethodChannel(name: METHOD_CHANNEL, binaryMessenger: registrar.messenger())
        registrar.addMethodCallDelegate(instance, channel: methodChannel)
        
        [EVENT_CHANNEL_Deeplink,
         EVENT_CHANNEL_DeferredDeeplink,
         EVENT_CHANNEL_DidPrintLog,
         EVENT_CHANNEL_PushLocalCallback,
         EVENT_CHANNEL_PushRemoteCallback,
         EVENT_CHANNEL_DidFetchInAppMessage,
         EVENT_CHANNEL_OnReceiveInAppMessageClick]
            .forEach {
                FlutterEventChannel(name: $0, binaryMessenger: registrar.messenger())
                    .setStreamHandler(AdBrixEventChannel.shared)
            }
    }
    
    //MethodChannel은 FlutterPlugin에 종속적이여서 이벤트채널처럼 다른 클래스로 분리하지 않았음
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        let methodName : String = call.method
        // event Controller
        
        switch methodName {
        case "setEventUploadCountInterval" :
            setEventUploadCountInterval (call)
        case"setEventUploadTimeInterval" :
            setEventUploadTimeInterval (call)
        case "gdprForgetMe" :
            gdprForgetMe ()
        case "setAge":
            setAge (call)
        case "setGender":
            setGender (call)
        case "setUserProperties" :
            setUserProperties (call)
        case "setRegistrationId":
            setRegistrationId(call)
        case "event" :
            events (call)
        case "login" :
            login (call)
        case "logout" :
            logout()
        case "commonSignUp":
            commonSignUp (call)
        case "appUpdate" :
            commonAppUpdate (call)
        case "userInvite" :
            commonUserInvite (call)
        case "useCredit" :
            commonUseCredit (call)
        case "gameTutorialComplete" :
            gameTutorialComplete (call)
        case "gameCharacterCreated" :
            gameCharacterCreated (call)
        case "gameStageCleared" :
            gameStageCleared (call)
        case "gameLevelAchieved":
            gameLevelAchieved (call)
        case "commerceViewHome" :
            commerceViewHome (call)
        case "commerceProductView" :
            commerceProductView (call)
        case "commerceAddToWishList" :
            commerceAddToWishList (call)
        case "commerceShare" :
            commerceShare (call)
        case "commercePaymentInfoAdd" :
            commercePaymentInfoAdd (call)
        case "commonPurchase" :
            commonPurchase (call)
        case "commerceCategoryView" :
            commerceCategoryView(call)
        case "commerceAddToCart" :
            commerceAddToCart (call)
        case "commerceReviewOrder" :
            commerceReviewOrder (call)
        case "commerceRefund" :
            commerceRefund (call)
        case "commerceSearch" :
            commerceSearch (call)
        case "commerceListView" :
            commerceListView (call)
        case "commerceCartView" :
            commerceCartView (call)
        case "startGettingIDFA" :
            AdbrixrmFlutterPlugin.startGettingIDFA()
        case "stopGettingIDFA" :
            AdbrixrmFlutterPlugin.stopGettingIDFA()
        case "setKakaoId" :
            setKakaoId(call)
        case "setCiProperties" :
            setCiProperties(call)
        case "flushAllEvents":
            flushAllEvents(result)
        case "setPushEnable":
            setPushEnable(call)
        case "openPush" :
            openPush(call)
        case "registeriOSLocalPushNotification" :
            registeriOSLocalPushNotification(call, result)
        case "cancelLocalPushNotification" :
            cancelLocalPushNotification(call)
        case "getPushEventList" :
            getPushEventList(call, result)
        case "deleteUserDataAndStopSDK" :
            deleteUserDataAndStopSDK(call, result)
        case "restartSDK":
            restartSDK(call, result)
        case "getSdkVersion":
            getSdkVersion(result)
        case "fetchActionHistoryByUserId":
            fetchActionHistoryByUserId(call, result)
        case "fetchActionHistoryByAdid":
            fetchActionHistoryByAdid(call, result)
        case "insertPushData":
            insertPushData(call)
        case "getActionHistory":
            getActionHistory(call, result)
        case "getAllActionHistory":
            getAllActionHistory(call, result)
        case "deleteActionHistory":
            deleteActionHistory(call, result)
        case "deleteAllActionHistoryByUserId":
            deleteAllActionHistoryByUserId(call, result)
        case "deleteAllActionHistoryByAdid":
            deleteAllActionHistoryByAdid(call, result)
        case "clearSyncedActionHistoryInLocalDB":
            clearSyncedActionHistoryInLocalDB(result)
        case "setInAppMessageFetchMode":
            setInAppMessageFetchMode(call)
        case "setInAppMessageToken":
            setInAppMessageToken(call)
        case "fetchInAppMessage":
            fetchInAppMessage(result)
        case "getAllInAppMessage":
            getAllInAppMessage(result)
        case "openInAppMessage":
            openInAppMessage(call, result)
        default:
            print("AdBrixRm Flutter plugin")
        }
    }
}


//MARK: - MethodHandler
extension AdbrixrmFlutterPlugin {
    
    func setRegistrationId(_ call: FlutterMethodCall) {
        guard let token = call.arguments as? String else {
            return
        }
        let data = Data(token.utf8)
        adBrix.setRegistrationId(deviceToken: data)
    }
    func setEventUploadCountInterval (_ call: FlutterMethodCall) {
        
        guard let arg = call.arguments as? String else {
            return
        }
        
        switch arg {
        case "MIN":
            adBrix.setEventUploadCountInterval(AdBrixRM.AdBrixEventUploadCountInterval.MIN)
        case "NORMAL" :
            adBrix.setEventUploadCountInterval(AdBrixRM.AdBrixEventUploadCountInterval.NORMAL)
        case "MAX" :
            adBrix.setEventUploadCountInterval(AdBrixRM.AdBrixEventUploadCountInterval.MAX)
        default:
            print("AdBrixRm Flutter plugin")
        }
    }

    func setEventUploadTimeInterval (_ call: FlutterMethodCall) {
        
        guard let arg = call.arguments as? String else {
            return
        }
        print(arg)
        switch arg {
        case "MIN":
            adBrix.setEventUploadTimeInterval(AdBrixRM.AdBrixEventUploadTimeInterval.MIN)
        case "NORMAL" :
            adBrix.setEventUploadTimeInterval(AdBrixRM.AdBrixEventUploadTimeInterval.NORMAL)
        case "MAX" :
            adBrix.setEventUploadTimeInterval(AdBrixRM.AdBrixEventUploadTimeInterval.MAX)
        default:
            print("AdBrixRm Flutter plugin")
        }
    }
    
    func gdprForgetMe () {
        
        adBrix.gdprForgetMe()
    }
    
    func setAge (_ call : FlutterMethodCall) {
        
        guard let arg = call.arguments as? Int else {
            return
        }
        adBrix.setAge(int: arg)
    }
    
    func setKakaoId(_ call : FlutterMethodCall){
        
        guard let kakaoId = call.arguments as? String else {
            return
        }
        adBrix.setKakaoId(kakaoId: kakaoId)
        
    }
    
    func setGender (_ call : FlutterMethodCall) {
        
        guard let arg = call.arguments as? String else {
            return
        }
        switch arg {
        case "FEMALE":
            adBrix.setGender(adBrixGenderType: AdBrixRM.AdBrixGenderType.Female)
        case "MALE" :
            adBrix.setGender(adBrixGenderType: AdBrixRM.AdBrixGenderType.Male)
        case "UNKNOWN" :
            adBrix.setGender(adBrixGenderType: AdBrixRM.AdBrixGenderType.Unknown)
        default:
            print("AdBrixRm Flutter plugin")
        }
    }
    
    func setUserProperties (_ call : FlutterMethodCall) {
        
        guard let arg = call.arguments as? [String: Any] else {
            return
        }
        let userProperties = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: arg)
        adBrix.setUserPropertiesWithAttr(attrModel: userProperties)
        
    }
    
    func setCiProperties (_ call : FlutterMethodCall) {
        
        guard let arg = call.arguments as? [String: Any] else {
            return
        }
        let ciProperties = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: arg)
        adBrix.setUserCiWithAttr(attrModel: ciProperties)
    }
    
    func events (_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
        let eventName = args["eventName"] as? String else {
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.eventWithAttr(eventName: eventName, value: eventAttr)
        } else {
            adBrix.event(eventName: eventName)
        }
    }
    
    func login (_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let userId = args["userId"] as? String else {
            return
        }
        
        adBrix.login(userId: userId)
        
    }
    
    func logout (){
        adBrix.logout();
    }
    
    func commonSignUp(_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let channel = args["channel"] as? Int,
              let adbrixChannel = AdBrixRM.AdBrixRmSignUpChannel(rawValue: channel) else {
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commonSignUpWithAttr(channel: adbrixChannel, commonAttr: eventAttr)
        } else {
            adBrix.commonSignUp(channel: adbrixChannel)
            
        }
    }
    
    func commonAppUpdate(_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let preVersion = args["preVersion"] as? String,
              let currVersion = args["currVersion"] as? String else {
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commonAppUpdateWithAttr(prev_ver: preVersion, curr_ver: currVersion, commonAttr: eventAttr)
        }else {
            adBrix.commonAppUpdate(prev_ver: preVersion, curr_ver: currVersion)
            
        }
    }
    
    func commonUserInvite(_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let channel = args["inviteChannel"] as? Int,
              let adbrixChannel = AdBrixRM.AdBrixRmInviteChannel(rawValue: channel)
        else {
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commonInviteWithAttr(channel: adbrixChannel, commonAttr: eventAttr)
        } else {
            adBrix.commonInvite(channel: adbrixChannel)
            
        }
    }
    
    func commonUseCredit(_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any] else {
            adBrix.commonUseCredit()
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commonUseCreditWithAttr(commonAttr: eventAttr)
        }
    }
    
    func gameTutorialComplete(_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let isSkip = args["isSkip"] as? Bool else {
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.gameTutorialCompletedWithAttr(isSkip: isSkip, gameInfoAttr: eventAttr)
        } else {
            adBrix.gameTutorialCompleted(isSkip: isSkip)
        }
    }
    
    func gameCharacterCreated(_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any] else {
            adBrix.gameCharacterCreated()
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.gameCharacterCreatedWithAttr(gameInfoAttr: eventAttr)
        }
    }
    
    func gameStageCleared(_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let stageName = args["stageName"] as? String else {
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.gameStageClearedWithAttr(stageName: stageName, gameInfoAttr: eventAttr)
        } else {
            adBrix.gameStageCleared(stageName: stageName)
        }
    }
    
    func gameLevelAchieved(_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let levelAchieved = args["levelAchieved"] as? Int else {
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.gameLevelAchievedWithAttr(level: levelAchieved, gameInfoAttr: eventAttr)
        } else {
            adBrix.gameLevelAchieved(level: levelAchieved)
        }
    }
    
    func commerceViewHome(_ call: FlutterMethodCall) {
        guard let args = call.arguments as? [String : Any] else {
            adBrix.commerceViewHome()
            return
        }

        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commerceViewHomeWithAttr(orderAttr: eventAttr)
        }
    }
    
    func commerceProductView(_ call : FlutterMethodCall) {
        guard let args = call.arguments as? [String : Any],
              let productModel = args["productModel"] as? [String: Any] else {
            return
        }
        
        let productResult = AdBrixUtility.AdBrixRmMakeProductModel(productModel: productModel)
        
        if let attr = args["attr"] as? [String: Any]{
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commerceProductViewWithAttr(productInfo: productResult, orderAttr: eventAttr)
            
        } else {
            adBrix.commerceProductView(productInfo:productResult)
            
        }
    }
    
    func commerceAddToWishList(_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let productModel = args["productModel"] as? [String: Any] else {
            return
        }
        
        let productResult = AdBrixUtility.AdBrixRmMakeProductModel(productModel: productModel)
        
        if let attr = args["attr"] as? [String: Any]{
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commerceAddToWishListWithAttr(productInfo: productResult, orderAttr: eventAttr)
        } else {
            adBrix.commerceAddToWishList(productInfo: productResult)
        }
    }
    
    func commerceShare (_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let productModel = args["productModel"] as? [String: Any],
              let shareChannel = args["sharingChannel"] as? String else {
            return
        }
        
        
        let productResult = AdBrixUtility.AdBrixRmMakeProductModel(productModel: productModel)
        
        var sharingChannel: AdBrixRM.AdBrixRmSharingChannel?
        switch shareChannel {
        case "FACEBOOK":
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingFacebook
        case "KAKAOTALK" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingKakaoTalk
        case "KAKAOSTORY" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingKakaoStory
        case "LINE" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingLine
        case "WHATSAPP" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingWhatsApp
        case "QQ" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingQQ
        case "WECHAT" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingWeChat
        case "SMS" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingSMS
        case "EMAIL" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingEmail
        case "COPYURL" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingCopyUrl
        case "ETC" :
            sharingChannel = AdBrixRM.AdBrixRmSharingChannel.AdBrixRmSharingETC
        default:
            sharingChannel = nil
            debugPrint("peterworks :::::: there is something wrong with shareChannel")
        }
        
        guard let _sharingChannel = sharingChannel else {return}
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceShareWithAttr(channel: _sharingChannel, productInfo: productResult, orderAttr: eventAttr)
            
        }  else {
            adBrix.commerceShare(channel: _sharingChannel, productInfo: productResult)
            
        }
        
    }
    
    func commercePaymentInfoAdd (_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any] else {
            adBrix.commercePaymentInfoAdded()
            return
        }
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            adBrix.commercePaymentInfoAddedWithAttr(paymentInfoAttr: eventAttr)
        }
    }
    
    func commonPurchase (_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let orderId = args["orderId"] as? String,
              let orderSale = args["orderSale"] as? Double,
              let productList = args["productList"] as? [[String: Any]],
              let discount = args["discount"] as? Double,
              let deliveryCharge = args["deliveryCharge"] as? Double,
              let paymentMethodString = args["paymentMethod"] as? String else {
            return
        }
        
        var paymentMethod: AdBrixRM.AdbrixRmPaymentMethod?
        
        switch paymentMethodString {
        case "CreditCard":
            paymentMethod = AdBrixRM.AdbrixRmPaymentMethod.CreditCard
        case "BankTransfer" :
            paymentMethod = AdBrixRM.AdbrixRmPaymentMethod.BankTransfer
        case "MobilePayment" :
            paymentMethod = AdBrixRM.AdbrixRmPaymentMethod.MobilePayment
        case "ETC" :
            paymentMethod = AdBrixRM.AdbrixRmPaymentMethod.ETC
        default:
            paymentMethod = nil
            debugPrint("peterworks :::::: there is something wrong with payment")
        }
        
        guard let paymentMethod = paymentMethod else {return}
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        
        if let attr = args["attr"] as? [String: Any] {
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commonPurchaseWithAttr(orderId: orderId, productInfo: productListArr, orderSales: orderSale, discount: discount, deliveryCharge: deliveryCharge, paymentMethod: paymentMethod, orderAttr: eventAttr)
            
        } else {
            
            adBrix.commonPurchase(orderId: orderId, productInfo: productListArr, orderSales: orderSale, discount: discount, deliveryCharge: deliveryCharge, paymentMethod: paymentMethod)
            
        }
        
    }
    
    func commerceCategoryView(_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let productList = args["productList"] as? [[String: Any]],
              let categoryList = args["categoryModel"] as? [String] else {
            return
        }
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        let myCategory = AdBrixUtility.AdBrixRmMakeProductCategory(categoryList: categoryList)
        
        if let attr = args["attr"] as? [String: Any] {
            
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceCategoryViewWithAttr(category: myCategory, productInfo: productListArr, orderAttr: eventAttr)
            
        } else {
            
            adBrix.commerceCategoryView(category: myCategory, productInfo: productListArr)
        }
        
    }
    
    func commerceAddToCart(_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let productList = args["productList"] as? [[String: Any]] else {
            return
        }
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        
        if let attr = args["attr"] as? [String: Any] {
            
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceAddToCartWithAttr(productInfo: productListArr, orderAttr: eventAttr)
            
        } else {
            
            adBrix.commerceAddToCart(productInfo: productListArr)
        }
    }
    
    func commerceReviewOrder (_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String: Any],
              let orderId = args["orderId"] as? String,
              let productList = args["productList"] as? [[String: Any]],
              let discount = args["discount"] as? Double,
              let deliveryCharge = args["deliveryCharge"] as? Double else {
            return
        }
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        
        if let attr = args["attr"] as? [String: Any]{
            
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceReviewOrderWithAttr(orderId: orderId, productInfo: productListArr, discount: discount, deliveryCharge: deliveryCharge, orderAttr: eventAttr)
            
        } else {
            
            adBrix.commerceReviewOrder(orderId: orderId, productInfo: productListArr, discount: discount, deliveryCharge: deliveryCharge)
            
        }
        
    }
    
    func commerceRefund (_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let orderId = args["orderId"] as? String,
              let productList = args["productList"] as? [[String: Any]],
              let penaltyCharge = args["penaltyCharge"] as? Double else {
            return
        }
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        
        if let attr = args["attr"] as? [String: Any] {
            
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceRefundWithAttr(orderId: orderId, productInfo: productListArr, penaltyCharge: penaltyCharge, orderAttr: eventAttr)
            
        } else {
            
            adBrix.commerceRefund(orderId: orderId, productInfo: productListArr, penaltyCharge: penaltyCharge)
            
        }
        
    }
    
    func commerceSearch(_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let keyword = args["keyword"] as? String,
              let productList = args["productList"] as? [[String: Any]] else {
            return
        }
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        
        if let attr = args["attr"] as? [String: Any] {
            
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceSearchWithAttr(productInfo: productListArr, keyword: keyword, orderAttr: eventAttr)
            
        } else {
            adBrix.commerceSearch(productInfo: productListArr, keyword: keyword)
        }
    }
    
    func commerceListView(_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let productList = args["productList"] as? [[String: Any]] else {
            return
        }
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        
        if let attr = args["attr"] as? [String: Any] {
            
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceListViewWithAttr(productInfo: productListArr, orderAttr: eventAttr)
            
        } else {
            adBrix.commerceListView(productInfo: productListArr)
        }
    }
    
    func commerceCartView (_ call : FlutterMethodCall) {
        
        guard let args = call.arguments as? [String : Any],
              let productList = args["productList"] as? [[String: Any]] else {
            return
        }
        
        let productListArr = AdBrixUtility.AdBrixRmMakeProductList(productList: productList)
        
        if let attr = args["attr"] as? [String: Any] {
            
            let eventAttr = AdBrixUtility.AdBrixRmAttrModelMakeAttrModel(attr: attr)
            
            adBrix.commerceCartViewWithAttr(productInfo: productListArr, orderAttr: eventAttr)
        } else {
            adBrix.commerceCartView(productInfo: productListArr)
        }
    }
    
    func flushAllEvents(_ result: @escaping FlutterResult) {
        adBrix.flushAllEvents() { dfnResult in
            if dfnResult.isSuccess {
                result(true)
            } else {
                result(false)
            }
        }
    }
    
    func setPushEnable(_ call : FlutterMethodCall) {
        guard let isPushEnable = call.arguments as? Bool else {
            NSLog("abxrm \(#function) [error] :: cannot parse arguments!, argument = \(call.arguments.debugDescription)")
            return
        }
        
        adBrix.setPushEnable(toPushEnable: isPushEnable)
    }
    
    func openPush(_ call : FlutterMethodCall) {
        guard let remotePushModelDic = call.arguments as? [String : Any] else{
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        guard let campaignId = remotePushModelDic["abx:gf:campaign_id"] as? String,
              let campaignRevisionNo = remotePushModelDic["abx:gf:campaign_id"] as? Int,
              let stepId = remotePushModelDic["abx:gf:campaign_id"] as? String,
              let cycleTime = remotePushModelDic["abx:gf:campaign_id"] as? String else {
            NSLog("abxrm \(#function) [error] :: cannot parse remotePushModelDic, remotePushModelDic = \(remotePushModelDic)")
            return
        }
        
        do {
            let abxRemotePushModel = try AbxRemotePushModel(campaignId: campaignId, campaignRevisionNo: campaignRevisionNo, stepId: stepId, cycleTime: cycleTime)
            adBrix.openPush(abxRemotePushModel)
        } catch let error {
            NSLog("abxrm \(#function) [error] :: can not get AbxRemotePushModel, error : \(error.localizedDescription)")
        }
    }
    
    func registeriOSLocalPushNotification(_ call : FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let localPushModelDic = call.arguments as? [String : Any] else{
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            result(FlutterError.init(code: self.ERROR_CODE_SDK_ERROR, message: "cannot parse flutter arguments", details: nil))
            return
        }
        
        let categoryId = localPushModelDic["categoryId"] as? String
        let threadId = localPushModelDic["threadId"] as? String
        let subtitle = localPushModelDic["subtitle"] as? String
        let imageUrlString = localPushModelDic["imageUrlString"] as? String
        
        var imageUrl : URL? = nil
        if let _imageUrlString = imageUrlString {
            imageUrl = URL(string: _imageUrlString)
        }
        
        guard let id = localPushModelDic["id"] as? String,
              let delaySeconds = localPushModelDic["delaySeconds"] as? Int,
              let title = localPushModelDic["title"] as? String,
              let body = localPushModelDic["body"] as? String,
              let badgeNumber = localPushModelDic["badgeNumber"] as? NSNumber else {
            NSLog("abxrm \(#function) [error] :: cannot parse localPushModelDic, localPushModelDic = \(localPushModelDic)")
            return
        }
        
        let calendar = Calendar.current
        guard let alarmTime = calendar.date(byAdding: .second, value: delaySeconds, to: Date()) else {
            result(FlutterError.init(code: self.ERROR_CODE_SDK_ERROR, message: "cannot parse delaySeconds to Date", details: nil))
            return
        }
        
        adBrix.registerLocalPushNotification(id: id, date: alarmTime, title: title, subtitle: subtitle, body: body, sound: nil, categoryId: categoryId, threadId: threadId, badgeNumber: badgeNumber, image: imageUrl, attrDic: nil, completionHandler: {(isRegSucc,error,pushid) in
            if isRegSucc {
                NSLog("abxrm [Push] :: registerLocalPushnotification is RegSucc:: \(isRegSucc) : \(pushid)")
                result(true)
            } else {
                NSLog("abxrm [Push] :: registerLocalPushnotification is RegSucc:: \(isRegSucc) : \(pushid)")
                result(FlutterError.init(code: self.ERROR_CODE_SDK_ERROR, message: "cannot parse delaySeconds to Date", details: nil))
            }
        })
    }
    
    func cancelLocalPushNotification(_ call : FlutterMethodCall) {
        guard let eventId = call.arguments as? [String] else{
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        adBrix.cancelLocalPushNotification(ids: eventId)
    }
    
    func getPushEventList(_ call : FlutterMethodCall, _ result: @escaping FlutterResult) {
        adBrix.getRegisteredLocalPushNotification() { pushList in
            result(pushList.description)
        }
    }
    
    func deleteUserDataAndStopSDK(_ call : FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let userId = call.arguments as? String else {
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        adBrix.deleteUserDataAndStopSDK(userId) { completion in
            switch completion {
            case .success:
                result(true)
            case .fail:
                result(false)
            @unknown default:
                result(FlutterError.init(code: self.ERROR_CODE_SDK_ERROR, message: "unkown Completion Type", details: nil))
            }
        }
    }
    
    func restartSDK(_ call : FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let userId = call.arguments as? String else {
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        adBrix.restartSDK(userId) { completion in
            switch completion {
            case .success:
                result(true)
            case .fail:
                result(false)
            @unknown default:
                result(FlutterError.init(code: self.ERROR_CODE_SDK_ERROR, message: "unkown Completion Type", details: nil))
            }
        }
    }
    
    func getSdkVersion(_ result: @escaping FlutterResult) {
        let sdkVersion = adBrix.getSDKVersion()
        result(sdkVersion)
    }
    
    func fetchActionHistoryByUserId(_ call : FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let fetchActionHistoryDic = call.arguments as? [String : Any] else{
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        let token = fetchActionHistoryDic["token"] as? String
        
        guard let actionType = fetchActionHistoryDic["actionType"] as? [String] else {
            NSLog("abxrm \(#function) [error] :: cannot parse fetchActionHistoryDic, fetchActionHistoryDic = \(fetchActionHistoryDic)")
            return
        }
        
        adBrix.fetchActionHistoryByUserId(token: token, actionType: actionType) { acitonHistoryResult in
            if acitonHistoryResult.isSucceeded {
                result(acitonHistoryResult.getData()?.debugDescription)
            } else {
                result(FlutterError.init(code: self.ERROR_CODE_SDK_ERROR, message: acitonHistoryResult.getError()?.localizedDescription, details: nil))
            }
        }
    }
    
    func fetchActionHistoryByAdid(_ call : FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let fetchActionHistoryDic = call.arguments as? [String : Any] else{
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        let token = fetchActionHistoryDic["token"] as? String
        
        guard let actionType = fetchActionHistoryDic["actionType"] as? [String] else {
            NSLog("abxrm \(#function) [error] :: cannot parse fetchActionHistoryDic, fetchActionHistoryDic = \(fetchActionHistoryDic)")
            return
        }
        
        adBrix.fetchActionHistoryByAdid(token: token, actionType: actionType) { acitonHistoryResult in
            if acitonHistoryResult.isSucceeded {
                result(acitonHistoryResult.getData()?.debugDescription)
            } else {
                result(FlutterError.init(code: self.ERROR_CODE_SDK_ERROR, message: acitonHistoryResult.getError()?.localizedDescription, details: nil))
            }
        }
    }
    
    func insertPushData(_ call : FlutterMethodCall) {
        guard let map = call.arguments as? [String : Any] else {
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        adBrix.insertPushData(pushJsonDic: map)
    }
    
    func getActionHistory(_ call : FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let getActionHistoryDic = call.arguments as? [String : Any] else{
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        guard let skip = getActionHistoryDic["skip"] as? Int,
              let limit = getActionHistoryDic["limit"] as? Int,
              let actionType = getActionHistoryDic["actionType"] as? [String] else {
            NSLog("abxrm \(#function) [error] :: cannot parse getActionHistoryDic, getActionHistoryDic = \(getActionHistoryDic)")
            return
        }
        
        adBrix.getActionHistory(skip: skip, limit: limit, actionType: actionType) { acitonHistoryResult in
            if acitonHistoryResult.isSucceeded {
                result(acitonHistoryResult.getData()?.debugDescription)
            } else {
                result(FlutterError.init(code: self.ERROR_CODE_SDK_ERROR, message: acitonHistoryResult.getError()?.localizedDescription, details: nil))
            }
        }
    }
    
    func getAllActionHistory(_ call : FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let getActionHistoryDic = call.arguments as? [String : Any] else{
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        guard let actionType = getActionHistoryDic["actionType"] as? [String] else {
            NSLog("abxrm \(#function) [error] :: cannot parse getActionHistoryDic, getActionHistoryDic = \(getActionHistoryDic)")
            return
        }
        
        adBrix.getAllActionHistory(actionType: actionType) { acitonHistoryResult in
            if acitonHistoryResult.isSucceeded {
                let dataArray = acitonHistoryResult.getData() ?? []
                result(dataArray.map{ $0.toJsonString() }.description)
            } else {
                result(FlutterError.init(code: self.ERROR_CODE_SDK_ERROR, message: acitonHistoryResult.getError()?.localizedDescription, details: nil))
            }
        }
    }
    
    func deleteActionHistory(_ call : FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let deleteActionHistory = call.arguments as? [String : Any] else{
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        let token = deleteActionHistory["token"] as? String
        
        guard let historyId = deleteActionHistory["historyId"] as? String,
              let timestamp = deleteActionHistory["timestamp"] as? Int64 else {
            NSLog("abxrm \(#function) [error] :: cannot parse deleteActionHistory, deleteActionHistory = \(deleteActionHistory)")
            return
        }
        
        adBrix.deleteActionHistory(token: token, historyId: historyId, timeStamp: timestamp) { acitonHistoryResult in
            if acitonHistoryResult.isSucceeded {
                result(true)
            } else {
                result(false)
            }
        }
    }
    
    func deleteAllActionHistoryByUserId(_ call : FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let deleteActionHistory = call.arguments as? [String : Any] else{
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        let token = deleteActionHistory["token"] as? String
        
        adBrix.deleteAllActionHistoryByUserId(token: token) { acitonHistoryResult in
            if acitonHistoryResult.isSucceeded {
                result(true)
            } else {
                result(false)
            }
        }
    }
    
    func deleteAllActionHistoryByAdid(_ call : FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let deleteActionHistory = call.arguments as? [String : Any] else{
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        let token = deleteActionHistory["token"] as? String
        
        adBrix.deleteAllActionHistoryByAdid(token: token) { acitonHistoryResult in
            if acitonHistoryResult.isSucceeded {
                result(true)
            } else {
                result(false)
            }
        }
    }
    
    func clearSyncedActionHistoryInLocalDB(_ result: @escaping FlutterResult) {
        adBrix.clearSyncedActionHistoryInLocalDB() { acitonHistoryResult in
            if acitonHistoryResult.isSucceeded {
                result(true)
            } else {
                result(false)
            }
        }
    }
    
    func setInAppMessageFetchMode(_ call : FlutterMethodCall) {
        guard let arg = call.arguments as? [String: Any?],
              let mode = arg["mode"] as? Int else{
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        guard let fetchMode = DfnInAppMessageFetchMode.init(rawValue: mode) else {
            NSLog("abxrm \(#function) [error] ::  cannot connvert \(mode) to DfnInAppMessageFetchMode")
            return
        }
        
        adBrix.setInAppMessageFetchMode(mode: fetchMode)
    }
    
    func setInAppMessageToken(_ call : FlutterMethodCall) {
        guard let arg = call.arguments as? [String: Any?],
              let token = arg["token"] as? String else {
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        adBrix.setInAppMessageToken(token: token)
    }
    
    func fetchInAppMessage(_ result: @escaping FlutterResult) {
        adBrix.fetchInAppMessage() { fetchResult in
            if fetchResult.isSucceeded {
                result(true)
            } else {
                result(false)
            }
        }
    }
    
    func getAllInAppMessage(_ result: @escaping FlutterResult) {
        adBrix.getAllInAppMessage() { fetchResult in
            if fetchResult.isSucceeded {
                let dataArray = fetchResult.getData() ?? []
                result(dataArray.map{ $0.campaignId }.description)
            } else {
                result(FlutterError.init(code: self.ERROR_CODE_SDK_ERROR, message: fetchResult.getError()?.localizedDescription, details: nil))
            }
        }
    }
    
    func openInAppMessage(_ call : FlutterMethodCall, _ result: @escaping FlutterResult) {
        guard let arg = call.arguments as? [String: Any?],
              let campaignId = arg["campaignId"] as? String
        else {
            NSLog("abxrm \(#function) [error] ::  cannot parse arguments, argument = \(call.arguments.debugDescription)")
            return
        }
        
        adBrix.openInAppMessage(campaignId: campaignId) { completion in
            switch completion {
            case .success:
                result(true)
            case .fail:
                result(false)
            @unknown default:
                result(FlutterError.init(code: self.ERROR_CODE_SDK_ERROR, message: "unkown Completion Type", details: nil))
            }
        }
    }

}

