import 'dart:async';
import 'dart:ffi';
import 'dart:io';

import 'package:flutter/services.dart';

enum AdBrixCurrency {
  KR_KRW,
  US_USD,
  JP_JPY,
  EU_EUR,
  UK_GBP,
  CH_CNY,
  TW_TWD,
  HK_HKD,
  ID_IDR, //Indonesia
  IN_INR, //India
  RU_RUB, //Russia
  TH_THB, //Thailand
  VN_VND, //Vietnam
  MY_MYR //Malaysia
}

enum AdBrixEventUploadCountInterval { MIN, NORMAL, MAX }

enum AdBrixEventUploadTimeInterval { MIN, NORMAL, MAX }

enum AdBrixGender { FEMALE, MALE, UNKNOWN }

enum AdBrixInviteChannel {
  Kakao,
  Naver,
  Line,
  Google,
  Facebook,
  Twitter,
  WhatsApp,
  QQ,
  WeChat,
  ETC
}

enum AdBrixPaymentMethod { CreditCard, BankTransfer, MobilePayment, ETC }

class AdBrixRm {
  static const MethodChannel _methodChannel =
      const MethodChannel('com.adbrix.methodChannel');
  static const EventChannel _eventChannelDeeplink =
      const EventChannel('com.adbrix.eventChannel.deeplink');
  static const EventChannel _eventChannelDeferredDeeplink =
      const EventChannel('com.adbrix.eventChannel.deferredDeeplink');
  static const EventChannel _eventChannelPushLocalCallback =
      const EventChannel('com.adbrix.eventChannel.pushLocalCallBack');
  static const EventChannel _eventChannelPushRemoteCallback =
      const EventChannel('com.adbrix.eventChannel.pushRemoteCallback');
  static const EventChannel _eventChannelOnReceiveInAppMessageClick =
      const EventChannel('com.adbrix.eventChannel.onReceiveInAppMessageClick');
  static const EventChannel _eventChannelDidFetchInAppMessage =
      const EventChannel('com.adbrix.eventChannel.didFetchInAppMessage');
  static const EventChannel _eventChannelDidPrintLog =
      const EventChannel('com.adbrix.eventChannel.didPrintLog');

  static const String _ARGUMENT_Deeplink = 'com.adbrix.stream.deeplink';
  static const String _ARGUMENT_DeferredDeeplink =
      'com.adbrix.stream.deferredDeeplink';
  static const String _ARGUMENT_PushLocalCallback =
      'com.adbrix.stream.pushLocalCallBack';
  static const String _ARGUMENT_PushRemoteCallback =
      'com.adbrix.stream.pushRemoteCallback';
  static const String _ARGUMENT_OnReceiveInAppMessageClick =
      'com.adbrix.stream.onReceiveInAppMessageClick';
  static const String _ARGUMENT_DidFetchInAppMessage =
      'com.adbrix.stream.didFetchInAppMessage';
  static const String _ARGUMENT_DidPrintLog = 'com.adbrix.stream.didPrintLog';

  static void setDeeplinkListener(Function(String deeplinkString) callback) {
    _eventChannelDeeplink
        .receiveBroadcastStream(_ARGUMENT_Deeplink)
        .listen((event) {
      callback(event.toString());
    });
  }

  static void setDeferredDeeplinkListener(
      Function(String deferredDeeplinkString) callback) {
    _eventChannelDeferredDeeplink
        .receiveBroadcastStream(_ARGUMENT_DeferredDeeplink)
        .listen((event) {
      callback(event.toString());
    });
  }

  @Deprecated("Local notification no longer supported")
  static void setPushLocalCallbackListener(
      Function(String jsonString) callback) {
    _eventChannelPushLocalCallback
        .receiveBroadcastStream(_ARGUMENT_PushLocalCallback)
        .listen((event) {
      callback(event.toString());
    });
  }

  static void setPushRemoteCallbackListener(
      Function(String jsonString) callback) {
    _eventChannelPushRemoteCallback
        .receiveBroadcastStream(_ARGUMENT_PushRemoteCallback)
        .listen((event) {
      callback(event.toString());
    });
  }

  static void setOnReceiveInAppMessageClickListener(
      Function(String jsonString) callback) {
    _eventChannelOnReceiveInAppMessageClick
        .receiveBroadcastStream(_ARGUMENT_OnReceiveInAppMessageClick)
        .listen((event) {
      callback(event.toString());
    });
  }

  static void setDidFetchInAppMessageListener(
      Function(Bool isSucceeded) callback) {
    _eventChannelDidFetchInAppMessage
        .receiveBroadcastStream(_ARGUMENT_DidFetchInAppMessage)
        .listen((event) {
      callback(event);
    });
  }

  static void setDidPrintLogListener(Function(String logString) callback) {
    _eventChannelDidPrintLog
        .receiveBroadcastStream(_ARGUMENT_DidPrintLog)
        .listen((event) {
      callback(event.toString());
    });
  }

  static void commerceAddToCart(
      {required List<AdBrixRmCommerceProductModel> productList,
      Map<String, dynamic>? attr}) {
    List<Map<String, dynamic>> getProductList = [];

    for (int i = 0; i < productList.length; i++) {
      Map<String, dynamic> product = mappingProductModel(productList[i]);
      getProductList.add(product);
    }
    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'productList': getProductList,
        'attr': attr,
      };

      _methodChannel.invokeMethod('commerceAddToCart', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'productList': getProductList,
      };

      _methodChannel.invokeMethod('commerceAddToCart', params);
    }
  }

  static void commerceAddToWishList(
      {required AdBrixRmCommerceProductModel productModel,
      Map<String, dynamic>? attr}) {
    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'productModel': productModel.getProductModel(),
        'attr': attr
      };

      _methodChannel.invokeMethod('commerceAddToWishList', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'productModel': productModel.getProductModel()
      };

      _methodChannel.invokeMethod('commerceAddToWishList', params);
    }
  }

  static void commerceCartView(
      {required List<AdBrixRmCommerceProductModel> productList,
      Map<String, dynamic>? attr}) {
    List<Map<String, dynamic>> getProductList = [];

    for (int i = 0; i < productList.length; i++) {
      Map<String, dynamic> product = mappingProductModel(productList[i]);
      getProductList.add(product);
    }

    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'productList': getProductList,
        'attr': attr
      };

      _methodChannel.invokeMethod('commerceCartView', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'productList': getProductList
      };

      _methodChannel.invokeMethod('commerceCartView', params);
    }
  }

  static void commerceCategoryView(
      {required AdBrixRmCommerceCategoryModel categoryModel,
      required List<AdBrixRmCommerceProductModel> productList,
      Map<String, dynamic>? attr}) {
    List<Map<String, dynamic>> getProductList = [];

    for (int i = 0; i < productList.length; i++) {
      Map<String, dynamic> product = mappingProductModel(productList[i]);
      getProductList.add(product);
    }

    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'categoryModel': categoryModel.getCategoryList(),
        'productList': getProductList,
        'attr': attr,
      };

      _methodChannel.invokeMethod('commerceCategoryView', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'categoryModel': categoryModel.getCategoryList(),
        'productList': getProductList
      };

      _methodChannel.invokeMethod('commerceCategoryView', params);
    }
  }

  static void commerceListView(
      {required List<AdBrixRmCommerceProductModel> productList,
      Map<String, dynamic>? attr}) {
    List<Map<String, dynamic>> getProductList = [];

    for (int i = 0; i < productList.length; i++) {
      Map<String, dynamic> product = mappingProductModel(productList[i]);
      getProductList.add(product);
    }

    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'productList': getProductList,
        'attr': attr
      };

      _methodChannel.invokeMethod('commerceListView', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'productList': getProductList
      };

      _methodChannel.invokeMethod('commerceListView', params);
    }
  }

  static void commercePaymentInfoAdd({Map<String, dynamic>? attr}) {
    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{'attr': attr};

      _methodChannel.invokeMethod('commercePaymentInfoAdd', params);
    } else {
      _methodChannel.invokeMethod('commercePaymentInfoAdd');
    }
  }

  static void commerceProductView(
      {required AdBrixRmCommerceProductModel productModel,
      Map<String, dynamic>? attr}) {
    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'productModel': productModel.getProductModel(),
        'attr': attr
      };

      _methodChannel.invokeMethod('commerceProductView', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'productModel': productModel.getProductModel()
      };

      _methodChannel.invokeMethod('commerceProductView', params);
    }
  }

  static void commerceRefund(
      {required String orderId,
      required List<AdBrixRmCommerceProductModel> productList,
      required double penaltyCharge,
      Map<String, dynamic>? attr}) {
    List<Map<String, dynamic>> getProductList = [];

    for (int i = 0; i < productList.length; i++) {
      Map<String, dynamic> product = mappingProductModel(productList[i]);
      getProductList.add(product);
    }

    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'orderId': orderId,
        'productList': getProductList,
        'penaltyCharge': penaltyCharge,
        'attr': attr,
      };

      _methodChannel.invokeMethod('commerceRefund', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'orderId': orderId,
        'productList': getProductList,
        'penaltyCharge': penaltyCharge
      };

      _methodChannel.invokeMethod('commerceRefund', params);
    }
  }

  static void commerceReviewOrder(
      {required String orderId,
      required List<AdBrixRmCommerceProductModel> productList,
      required double discount,
      required double deliveryCharge,
      Map<String, dynamic>? attr}) {
    List<Map<String, dynamic>> getProductList = [];

    for (int i = 0; i < productList.length; i++) {
      Map<String, dynamic> product = mappingProductModel(productList[i]);
      getProductList.add(product);
    }

    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'orderId': orderId,
        'productList': getProductList,
        'discount': discount,
        'deliveryCharge': deliveryCharge,
        'attr': attr
      };

      _methodChannel.invokeMethod('commerceReviewOrder', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'orderId': orderId,
        'productList': getProductList,
        'discount': discount,
        'deliveryCharge': deliveryCharge,
      };

      _methodChannel.invokeMethod('commerceReviewOrder', params);
    }
  }

  static void commerceSearch(
      {required String keyword,
      required List<AdBrixRmCommerceProductModel> productList,
      Map<String, dynamic>? attr}) {
    List<Map<String, dynamic>> getProductList = [];

    for (int i = 0; i < productList.length; i++) {
      Map<String, dynamic> product = mappingProductModel(productList[i]);
      getProductList.add(product);
    }

    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'keyword': keyword,
        'productList': getProductList,
        'attr': attr
      };

      _methodChannel.invokeMethod('commerceSearch', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'keyword': keyword,
        'productList': getProductList
      };

      _methodChannel.invokeMethod('commerceSearch', params);
    }
  }

  static void commerceShare(
      {required AdBrixSharingChannel sharingChannel,
      required AdBrixRmCommerceProductModel productModel,
      Map<String, dynamic>? attr}) {
    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'sharingChannel': sharingChannel.toString().split('.').last,
        'productModel': productModel.getProductModel(),
        'attr': attr
      };

      _methodChannel.invokeMethod('commerceShare', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'sharingChannel': sharingChannel.toString().split('.').last,
        'productModel': productModel.getProductModel()
      };

      _methodChannel.invokeMethod('commerceShare', params);
    }
  }

  static void commerceViewHome({Map<String, dynamic>? attr}) {
    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{'attr': attr};

      _methodChannel.invokeMethod('commerceViewHome', params);
    } else {
      _methodChannel.invokeMethod('commerceViewHome');
    }
  }

  static void commonAppUpdate(
      {required String preVersion,
      required String currVersion,
      Map<String, dynamic>? attr}) {
    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'preVersion': preVersion,
        'currVersion': currVersion,
        'attr': attr
      };

      _methodChannel.invokeMethod('appUpdate', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'preVersion': preVersion,
        'currVersion': currVersion,
      };

      _methodChannel.invokeMethod('appUpdate', params);
    }
  }

  static void commonPurchase(
      {required String orderId,
      required List<AdBrixRmCommerceProductModel> productList,
      required double orderSale,
      required double discount,
      required double deliveryCharge,
      required AdBrixPaymentMethod paymentMethod,
      Map<String, dynamic>? attr}) {
    List<Map<String, dynamic>> getProductList = [];

    for (int i = 0; i < productList.length; i++) {
      Map<String, dynamic> product = mappingProductModel(productList[i]);
      getProductList.add(product);
    }

    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'orderId': orderId,
        'productList': getProductList,
        'orderSale': orderSale,
        'discount': discount,
        'deliveryCharge': deliveryCharge,
        'paymentMethod': paymentMethod.toString().split('.').last,
        'attr': attr,
      };

      _methodChannel.invokeMethod('commonPurchase', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'orderId': orderId,
        'productList': getProductList,
        'orderSale': orderSale,
        'discount': discount,
        'deliveryCharge': deliveryCharge,
        'paymentMethod': paymentMethod.toString().split('.').last
      };

      _methodChannel.invokeMethod('commonPurchase', params);
    }
  }

  static void commonSignUp(
      {required AdBrixSignUpChannel channel, Map<String, dynamic>? attr}) {
    int channelValue = 0;
    String channelString = channel.toString().split('.').last;

    switch (channelString) {
      case "Kakao":
        {
          channelValue = 1;
        }
        break;
      case "Naver":
        {
          channelValue = 2;
        }
        break;
      case "Line":
        {
          channelValue = 3;
        }
        break;
      case "Google":
        {
          channelValue = 4;
        }
        break;
      case "Facebook":
        {
          channelValue = 5;
        }
        break;
      case "Twitter":
        {
          channelValue = 6;
        }
        break;
      case "WhatsApp":
        {
          channelValue = 7;
        }
        break;
      case "QQ":
        {
          channelValue = 8;
        }
        break;
      case "WeChat":
        {
          channelValue = 9;
        }
        break;
      case "UserId":
        {
          channelValue = 10;
        }
        break;
      case "ETC":
        {
          channelValue = 11;
        }
        break;
      case "SkTid":
        {
          channelValue = 12;
        }
        break;
      case "AppleId":
        {
          channelValue = 13;
        }
    }

    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'channel': channelValue,
        'attr': attr
      };

      _methodChannel.invokeMethod('commonSignUp', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'channel': channelValue,
      };

      _methodChannel.invokeMethod('commonSignUp', params);
    }
  }

  static void commonUseCredit({Map<String, dynamic>? attr}) {
    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{'attr': attr};

      _methodChannel.invokeMethod('useCredit', params);
    } else {
      _methodChannel.invokeMethod('useCredit');
    }
  }

  static void commonUserInvite(
      {required AdBrixInviteChannel inviteChannel,
      Map<String, dynamic>? attr}) {
    int channelValue = 0;
    String channelString = inviteChannel.toString().split('.').last;

    switch (channelString) {
      case "Kakao":
        {
          channelValue = 1;
        }
        break;
      case "Naver":
        {
          channelValue = 2;
        }
        break;
      case "Line":
        {
          channelValue = 3;
        }
        break;
      case "Google":
        {
          channelValue = 4;
        }
        break;
      case "Facebook":
        {
          channelValue = 5;
        }
        break;
      case "Twitter":
        {
          channelValue = 6;
        }
        break;
      case "WhatsApp":
        {
          channelValue = 7;
        }
        break;
      case "QQ":
        {
          channelValue = 8;
        }
        break;
      case "WeChat":
        {
          channelValue = 9;
        }
        break;
      case "ETC":
        {
          channelValue = 10;
        }
    }

    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'inviteChannel': channelValue,
        'attr': attr
      };

      _methodChannel.invokeMethod('userInvite', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'inviteChannel': channelValue
      };

      _methodChannel.invokeMethod('userInvite', params);
    }
  }

  @Deprecated("Use 'event()'")
  static void events(
      {required String eventName, Map<String, dynamic>? attr}) async {
    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'eventName': eventName,
        'attr': attr
      };
      _methodChannel.invokeMethod('event', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'eventName': eventName,
      };

      _methodChannel.invokeMethod('event', params);
    }
  }

  static void setRegistrationId({required String token}) async {
    _methodChannel.invokeMethod('setRegistrationId', token);
  }

  static void event(
      {required String eventName, Map<String, dynamic>? attr}) async {
    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'eventName': eventName,
        'attr': attr
      };
      _methodChannel.invokeMethod('event', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'eventName': eventName,
      };

      _methodChannel.invokeMethod('event', params);
    }
  }

  static void gameCharacterCreated({Map<String, dynamic>? attr}) {
    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'attr': attr,
      };

      _methodChannel.invokeMethod('gameCharacterCreated', params);
    } else {
      _methodChannel.invokeMethod('gameCharacterCreated');
    }
  }

  static void gameLevelAchieved(
      {required int levelAchieved, Map<String, dynamic>? attr}) {
    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'levelAchieved': levelAchieved,
        'attr': attr,
      };

      _methodChannel.invokeMethod('gameLevelAchieved', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'levelAchieved': levelAchieved
      };

      _methodChannel.invokeMethod('gameLevelAchieved', params);
    }
  }

  static void gameStageCleared({
    required String stageName,
    Map<String, dynamic>? attr,
  }) {
    if (attr != null) {
      _attrBoolChanger(attr);

      Map<String, dynamic> params = <String, dynamic>{
        'stageName': stageName,
        'attr': attr,
      };

      _methodChannel.invokeMethod('gameStageCleared', params);
    } else {
      Map<String, dynamic> params = <String, dynamic>{
        'stageName': stageName,
      };

      _methodChannel.invokeMethod('gameStageCleared', params);
    }
  }

  static void gameTutorialComplete(
      {required bool isSkip, Map<String, dynamic>? attr}) {
    if (attr != null) {
      Map<String, dynamic> param = <String, dynamic>{
        'isSkip': isSkip,
        'attr': attr
      };

      _methodChannel.invokeMethod('gameTutorialComplete', param);
    } else {
      Map<String, dynamic> param = <String, dynamic>{'isSkip': isSkip};

      _methodChannel.invokeMethod('gameTutorialComplete', param);
    }
  }

  static void gdprForgetMe() {
    _methodChannel.invokeMethod('gdprForgetMe');
  }

  static void login({required String userId}) {
    Map<String, dynamic> params = <String, dynamic>{
      'userId': userId,
    };

    _methodChannel.invokeMethod('login', params);
  }

  static void logout() {
    _methodChannel.invokeMethod('logout');
  }

  static Map<String, dynamic> mappingProductModel(
      AdBrixRmCommerceProductModel item) {
    Map<String, dynamic> dic = item.getProductModel();

    return dic;
  }

  static void setAge({required int age}) {
    _methodChannel.invokeMethod('setAge', age);
  }

  static void setCiProperties({required Map<String, dynamic> properties}) {
    _methodChannel.invokeMethod('setCiProperties', properties);
  }

  static void setEventUploadCountInterval(
      {required AdBrixEventUploadCountInterval interval}) {
    _methodChannel.invokeMethod(
        'setEventUploadCountInterval', interval.toString().split('.').last);
  }

  static void setEventUploadTimeInterval(
      {required AdBrixEventUploadTimeInterval interval}) {
    _methodChannel.invokeMethod(
        'setEventUploadTimeInterval', interval.toString().split('.').last);
  }

  static void setGender({required AdBrixGender gender}) {
    _methodChannel.invokeMethod('setGender', gender.toString().split('.').last);
  }

  static void setKakaoId({required String kakaoId}) {
    _methodChannel.invokeMethod('setKakaoId', kakaoId);
  }

  static void setUserProperties({required Map<String, dynamic> properties}) {
    _methodChannel.invokeMethod('setUserProperties', properties);
  }

  static void startGettingIDFA() {
    _methodChannel.invokeMethod('startGettingIDFA');
  }

  static void stopGettingIDFA() {
    _methodChannel.invokeMethod('stopGettingIDFA');
  }

  static Future<bool?> flushAllEvents() {
    return _methodChannel.invokeMethod<bool>('flushAllEvents');
  }

  static void setPushEnable({required bool isPushEnable}) {
    _methodChannel.invokeMethod('setPushEnable', isPushEnable);
  }

  static Future<bool?> deleteUserDataAndStopSDK({required String userId}) {
    return _methodChannel.invokeMethod<bool>(
        'deleteUserDataAndStopSDK', userId);
  }

  static Future<bool?> restartSDK({required String userId}) {
    return _methodChannel.invokeMethod<bool>('restartSDK', userId);
  }

  static Future<String?> getSdkVersion() {
    return _methodChannel.invokeMethod<String>('getSdkVersion');
  }

  // static Future<String?> fetchActionHistoryByUserId(
  //     {required String? token, required List<String> actionType}) {
  //   var argument = <String, dynamic>{"token": token, "actionType": actionType};

  //   return _methodChannel.invokeMethod<String>(
  //       'fetchActionHistoryByUserId', argument);
  // }

  // static Future<String?> fetchActionHistoryByAdid(
  //     {required String? token, required List<String> actionType}) {
  //   var argument = <String, dynamic>{"token": token, "actionType": actionType};

  //   return _methodChannel.invokeMethod<String>(
  //       'fetchActionHistoryByAdid', argument);
  // }

  // static Future<String?> getActionHistory(
  //     {required int skip,
  //     required int limit,
  //     required List<String> actionType}) {
  //   var argument = <String, dynamic>{
  //     "skip": skip,
  //     "limit": limit,
  //     "actionType": actionType
  //   };

  //   return _methodChannel.invokeMethod<String>('getActionHistory', argument);
  // }

  // static Future<String?> getAllActionHistory(
  //     {required List<String> actionType}) {
  //   var argument = <String, dynamic>{"actionType": actionType};

  //   return _methodChannel.invokeMethod<String>('getAllActionHistory', argument);
  // }

  // static Future<bool?> deleteActionHistory(
  //     {required String? token,
  //     required String historyId,
  //     required int timestamp}) {
  //   var argument = <String, dynamic>{
  //     "token": token,
  //     "historyId": historyId,
  //     "timestamp": timestamp
  //   };

  //   return _methodChannel.invokeMethod<bool>('deleteActionHistory', argument);
  // }

  // static Future<bool?> deleteAllActionHistoryByUserId(
  //     {required String? token}) {
  //   var argument = <String, dynamic>{"token": token};

  //   return _methodChannel.invokeMethod<bool>(
  //       'deleteAllActionHistoryByUserId', argument);
  // }

  // static Future<bool?> deleteAllActionHistoryByAdid({required String? token}) {
  //   var argument = <String, dynamic>{"token": token};

  //   return _methodChannel.invokeMethod<bool>(
  //       'deleteAllActionHistoryByAdid', argument);
  // }

  // static Future<bool?> clearSyncedActionHistoryInLocalDB() {
  //   return _methodChannel
  //       .invokeMethod<bool>('clearSyncedActionHistoryInLocalDB');
  // }

  static void setInAppMessageFetchMode(
      {required DfnInAppMessageFetchMode dfnInAppMessageFetchMode}) {
    int mode = 0;
    switch (dfnInAppMessageFetchMode) {
      case DfnInAppMessageFetchMode.USER_ID:
        mode = 0;
        break;
      case DfnInAppMessageFetchMode.ADID:
        mode = 1;
        break;
      default:
        mode = 0;
        break;
    }
    var argument = <String, dynamic>{"mode": mode};
    _methodChannel.invokeMethod('setInAppMessageFetchMode', argument);
  }

  static void setInAppMessageToken({required String token}) {
    var argument = <String, dynamic>{"token": token};
    _methodChannel.invokeMethod('setInAppMessageToken', argument);
  }

  static Future<bool?> fetchInAppMessage() {
    return _methodChannel.invokeMethod<bool>('fetchInAppMessage');
  }

  static Future<String?> getAllInAppMessage() {
    return _methodChannel.invokeMethod<String>('getAllInAppMessage');
  }

  static Future<bool?> openInAppMessage({required String campaignId}) {
    var argument = <String, dynamic>{"campaignId": campaignId};
    return _methodChannel.invokeMethod<bool>('openInAppMessage', argument);
  }

  // static void setBigTextClientPushEvent(
  //     BigTextPushProperties properties, bool alwaysIsShown) {
  //   if (Platform.isIOS) {
  //     return;
  //   }
  //   Map<String, dynamic> params = <String, dynamic>{
  //     'bigTextPushPropertiesJson': jsonEncode(properties.toMap()),
  //     'alwaysIsShown': alwaysIsShown,
  //   };
  //   _methodChannel.invokeMethod('setBigTextClientPushEvent', params);
  // }

  // static void setBigPictureClientPushEvent(
  //     BigPicturePushProperties properties, bool alwaysIsShown) {
  //   if (Platform.isIOS) {
  //     return;
  //   }
  //   Map<String, dynamic> params = <String, dynamic>{
  //     'bigPicturePushPropertiesJson': jsonEncode(properties.toMap()),
  //     'alwaysIsShown': alwaysIsShown,
  //   };
  //   _methodChannel.invokeMethod('setBigPictureClientPushEvent', params);
  // }

  // static Future<bool?>? registeriOSLocalPushNotification(
  //     DfniOSLocalPushParameter localPushParameter) {
  //   if (Platform.isAndroid) {
  //     return null;
  //   }

  //   Map<String, dynamic> arguments = localPushParameter.toMap();

  //   return _methodChannel.invokeMethod<bool>(
  //       'registeriOSLocalPushNotification', arguments);
  // }

  // static void cancelLocalPushNotification(List<String> eventIds) {
  //   var argument = <String, dynamic>{"eventId": eventIds};
  //   _methodChannel.invokeMethod('cancelLocalPushNotification', argument);
  // }

  // static Future<String?> getPushEventList() {
  //   return _methodChannel.invokeMethod<String>('getPushEventList', null);
  // }

  // static void setPushIconStyle(String smallIconName, String largeIconName) {
  //   Map<String, dynamic> params = <String, dynamic>{
  //     "smallIconName": smallIconName,
  //     "largeIconName": largeIconName
  //   };
  //   _methodChannel.invokeMethod('setPushIconStyle', params);
  // }

  // static void setNotificationOption(
  //     NotificationCompat priority, NotificationCompat visibility) {
  //   int priorityValue = 0;
  //   switch (priority) {
  //     case NotificationCompat.PRIORITY_DEFAULT:
  //       priorityValue = 0;
  //       break;
  //     case NotificationCompat.PRIORITY_HIGH:
  //       priorityValue = 1;
  //       break;
  //     case NotificationCompat.PRIORITY_LOW:
  //       priorityValue = -1;
  //       break;
  //     case NotificationCompat.PRIORITY_MAX:
  //       priorityValue = 2;
  //       break;
  //     case NotificationCompat.PRIORITY_MIN:
  //       priorityValue = -2;
  //       break;
  //     default:
  //       priorityValue = 0;
  //       break;
  //   }
  //   int visibilityValue = 1;
  //   switch (visibility) {
  //     case NotificationCompat.VISIBILITY_PRIVATE:
  //       visibilityValue = 0;
  //       break;
  //     case NotificationCompat.VISIBILITY_PUBLIC:
  //       visibilityValue = 1;
  //       break;
  //     case NotificationCompat.VISIBILITY_SECRET:
  //       visibilityValue = -1;
  //       break;
  //     default:
  //       visibilityValue = 1;
  //       break;
  //   }
  //   Map<String, dynamic> params = <String, dynamic>{
  //     "priority": priorityValue,
  //     "visibility": visibilityValue
  //   };
  //   _methodChannel.invokeMethod('setNotificationOption', params);
  // }

  static void setNotificationChannel(
      String channelName,
      String channelDescription) {

    if (!Platform.isAndroid) {
      return;
    }

    Map<String, dynamic> params = <String, dynamic>{
      "channelName": channelName,
      "channelDescription": channelDescription
    };
    _methodChannel.invokeMethod('setNotificationChannel', params);
  }

  static void _attrBoolChanger(Map<String, dynamic> attr) {
    if (Platform.isIOS) {
      attr.entries.forEach((e) {
        if (e.value.runtimeType == bool) {
          String boolValueString = e.value.toString();
          attr.update(e.key, (value) => boolValueString);
        }
      });
    }
  }
}

class AdBrixRmCommerceCategoryModel {
  String category1 = "";
  String category2 = "";
  String category3 = "";
  String category4 = "";
  String category5 = "";
  List<String> _getList = [];

  AdBrixRmCommerceCategoryModel(String category1, String? category2,
      String? category3, String? category4, String? category5) {
    if ((category2 != null) &&
        (category3 != null) &&
        (category4 != null) &&
        (category5 != null)) {
      this.category1 = category1;
      this.category2 = category2;
      this.category3 = category3;
      this.category4 = category4;
      this.category5 = category5;

      this.setCategoryList(
          [category1, category2, category3, category4, category5]);
    } else if ((category2 != null) &&
        (category3 != null) &&
        (category4 != null)) {
      this.category1 = category1;
      this.category2 = category2;
      this.category3 = category3;
      this.category4 = category4;

      this.setCategoryList([category1, category2, category3, category4]);
    } else if ((category2 != null) && (category3 != null)) {
      this.category1 = category1;
      this.category2 = category2;
      this.category3 = category3;

      this.setCategoryList([category1, category2, category3]);
    } else if ((category2 != null)) {
      this.category1 = category1;
      this.category2 = category2;

      this.setCategoryList([category1, category2]);
    } else {
      this.category1 = category1;

      this.setCategoryList([category1]);
    }
  }

  List<String> getCategoryList() {
    return _getList;
  }

  void setCategoryList(List<String> getList) {
    this._getList = getList;
  }

  static AdBrixRmCommerceCategoryModel create(
      {required String category1,
      String? category2,
      String? category3,
      String? category4,
      String? category5}) {
    return new AdBrixRmCommerceCategoryModel(
        category1, category2, category3, category4, category5);
  }
}

class AdBrixRmCommerceProductModel {
  String productId = "";
  String productName = "";
  double price = 0.0;
  double discount = 0.0;
  int quantity = 0;
  String currency = "";
  List<String> category = [];
  Map<String, dynamic> productAttr = {};

  Map<String, dynamic> _productParameter = {};

  AdBrixRmCommerceProductModel(
      {required String productId,
      required String productName,
      required double price,
      required double discount,
      required int quantity,
      required AdBrixCurrency currency,
      required AdBrixRmCommerceCategoryModel category,
      Map<String, dynamic>? productAttr}) {
    if (productAttr != null) {
      this.productId = productId;
      this.productName = productName;
      this.price = price;
      this.discount = discount;
      this.quantity = quantity;
      this.currency = currency.toString().split('.').last;
      this.category = category.getCategoryList();
      this.productAttr = productAttr;

      _attrBoolChanger(productAttr);

      Map<String, dynamic> setProduct = <String, dynamic>{
        'productId': productId,
        'productName': productName,
        'price': price,
        'discount': discount,
        'quantity': quantity,
        'currency': currency.toString().split('.').last,
        'category': category.getCategoryList(),
        'productAttr': productAttr
      };

      this.setProductModel(setProduct);
    } else {
      this.productId = productId;
      this.productName = productName;
      this.price = price;
      this.discount = discount;
      this.quantity = quantity;
      this.currency = currency.toString().split('.').last;
      this.category = category.getCategoryList();

      Map<String, dynamic> setProduct = <String, dynamic>{
        'productId': productId,
        'productName': productName,
        'price': price,
        'discount': discount,
        'quantity': quantity,
        'currency': currency.toString().split('.').last,
        'category': category.getCategoryList(),
      };

      this.setProductModel(setProduct);
    }
  }

  Map<String, dynamic> getProductModel() {
    return _productParameter;
  }

  void setProductModel(Map<String, dynamic> setupProduct) {
    this._productParameter = setupProduct;
  }

  static AdBrixRmCommerceProductModel create(
      {required String productId,
      required String productName,
      required double price,
      required double discount,
      required int quantity,
      required AdBrixCurrency currency,
      required AdBrixRmCommerceCategoryModel category,
      Map<String, dynamic>? productAttr}) {
    return new AdBrixRmCommerceProductModel(
        productId: productId,
        productName: productName,
        price: price,
        discount: discount,
        quantity: quantity,
        currency: currency,
        category: category,
        productAttr: productAttr);
  }

  static void _attrBoolChanger(Map<String, dynamic> attr) {
    if (Platform.isIOS) {
      attr.entries.forEach((e) {
        if (e.value.runtimeType == bool) {
          String boolValueString = e.value.toString();
          attr.update(e.key, (value) => boolValueString);
        }
      });
    }
  }
}

enum AdBrixSharingChannel {
  FACEBOOK,
  KAKAOTALK,
  KAKAOSTORY,
  LINE,
  WHATSAPP,
  QQ,
  WECHAT,
  SMS,
  EMAIL,
  COPYURL,
  ETC
}

enum AdBrixSignUpChannel {
  Kakao,
  Naver,
  Line,
  Google,
  Facebook,
  Twitter,
  WhatsApp,
  QQ,
  WeChat,
  UserId,
  ETC,
  SkTid,
  AppleId
}

enum DfnInAppMessageFetchMode { USER_ID, ADID }

enum NotificationCompat {
  PRIORITY_DEFAULT,
  PRIORITY_HIGH,
  PRIORITY_LOW,
  PRIORITY_MAX,
  PRIORITY_MIN,
  VISIBILITY_PRIVATE,
  VISIBILITY_PUBLIC,
  VISIBILITY_SECRET
}

enum NotificationManager {
  IMPORTANCE_DEFAULT,
  IMPORTANCE_HIGH,
  IMPORTANCE_LOW,
  IMPORTANCE_MAX,
  IMPORTANCE_MIN,
  IMPORTANCE_NONE,
  IMPORTANCE_UNSPECIFIED
}

class BigTextPushProperties {
  int second = 0;
  int eventId = 0;
  String title = "";
  String contentText = "";
  String summaryText = "";
  String bigContentTitle = "";
  String bigText = "";
  String deepLinkUri = "";

  BigTextPushProperties(this.second, this.eventId, this.title, this.contentText,
      this.summaryText, this.bigContentTitle, this.bigText, this.deepLinkUri);
  Map<String, dynamic> toMap() => {
        'second': second,
        'eventId': eventId,
        'title': title,
        'contentText': contentText,
        'summaryText': summaryText,
        'bigContentTitle': bigContentTitle,
        'bigText': bigText,
        'deepLinkUri': deepLinkUri
      };
}

class BigPicturePushProperties {
  int second = 0;
  int eventId = 0;
  int resourceId = 0;
  String title = "";
  String contentText = "";
  String summaryText = "";
  String bigContentTitle = "";
  String bigPictureUrl = "";
  String deepLinkUri = "";

  BigPicturePushProperties(
      this.second,
      this.eventId,
      this.resourceId,
      this.title,
      this.contentText,
      this.summaryText,
      this.bigContentTitle,
      this.bigPictureUrl,
      this.deepLinkUri);
  Map<String, dynamic> toMap() => {
        'second': second,
        'eventId': eventId,
        'resourceId': resourceId,
        'title': title,
        'contentText': contentText,
        'summaryText': summaryText,
        'bigContentTitle': bigContentTitle,
        'bigPictureUrl': bigPictureUrl,
        'deepLinkUri': deepLinkUri
      };
}

class DfniOSLocalPushParameter {
  String id = "";
  int delaySeconds = 0;
  String title = "";
  String? subtitle = "";
  String body = "";
  String? categoryId;
  String? threadId;
  int badgeNumber = 0;
  String? imageUrlString;

  DfniOSLocalPushParameter(
      this.id,
      this.delaySeconds,
      this.title,
      this.subtitle,
      this.body,
      this.categoryId,
      this.threadId,
      this.badgeNumber,
      this.imageUrlString);

  Map<String, dynamic> toMap() => {
        'id': id,
        'delaySeconds': delaySeconds,
        'title': title,
        'subtitle': subtitle,
        'body': body,
        'categoryId': categoryId,
        'threadId': threadId,
        'badgeNumber': badgeNumber,
        'imageUrlString': imageUrlString
      };
}
